(function outer(modules, cache, entries){

  /**
   * Global
   */

  var global = (function(){ return this; })();

  /**
   * Require `name`.
   *
   * @param {String} name
   * @api public
   */

  function require(name){
    if (cache[name]) return cache[name].exports;
    if (modules[name]) return call(name, require);
    throw new Error('cannot find module "' + name + '"');
  }

  /**
   * Call module `id` and cache it.
   *
   * @param {Number} id
   * @param {Function} require
   * @return {Function}
   * @api private
   */

  function call(id, require){
    var m = cache[id] = { exports: {} };
    var mod = modules[id];
    var name = mod[2];
    var fn = mod[0];
    var threw = true;

    try {
      fn.call(m.exports, function(req){
        var dep = modules[id][1][req];
        return require(dep || req);
      }, m, m.exports, outer, modules, cache, entries);
      threw = false;
    } finally {
      if (threw) {
        delete cache[id];
      } else if (name) {
        // expose as 'name'.
        cache[name] = cache[id];
      }
    }

    return cache[id].exports;
  }

  /**
   * Require all entries exposing them on global if needed.
   */

  for (var id in entries) {
    if (entries[id]) {
      global[entries[id]] = require(id);
    } else {
      require(id);
    }
  }

  /**
   * Duo flag.
   */

  require.duo = true;

  /**
   * Expose cache.
   */

  require.cache = cache;

  /**
   * Expose modules
   */

  require.modules = modules;

  /**
   * Return newest require.
   */

   return require;
})({
1: [function(require, module, exports) {



// Libs

var request = require('visionmedia/superagent@1.1.0');
var Arr = require('jmas/arr@0.1.4');


// Templates

var dialogTemplate = require('./templates/dialog.hogan');
var sourceItemTemplate = require('./templates/source-item.hogan');
var dialogSave = require('./templates/save-dialog.hogan');
var dialogFinish = require('./templates/finish-dialog.hogan');
var assetFolderTemplate = require('./templates/asset-folder.hogan');
var photoItemTemplate = require('./templates/photo-item.hogan');
var memeTemplate = require('./templates/meme.hogan');

// Varibles
var isHttps = (location.href.indexOf('https') === 0 ? true : false);
var sources = new Arr(
    {
        id: 'computer',
        name: 'Computer'
    },
    {
        id: 'backgrounds',
        name: 'piZap Backgrounds'
    },
    {
        id: 'webcam',
        name: 'Webcam'
    },
    {
        id: 'google',
        name: 'Google Search',
        libUrl: null
    },
    {
        id: 'facebook',
        name: 'Facebook',
        libUrl: null
    },
    {
        id: 'instagram',
        name: 'Instagram',
        libUrl: null,
        clientId: '40c8ce29a79547d484da9193d9fa181b'
    },
    {
        id: 'dropbox',
        name: 'Dropbox',
        libUrl: (isHttps ? 'https' : 'http') + '://www.dropbox.com/static/api/2/dropins.js',
        attrs: {
            'id': 'dropboxjs',
            'data-app-key': 'krklwcfzbe1m0f5'
        }
    },
    {
        id: 'google-drive',
        name: 'Google Drive',
        libUrl: (isHttps ? 'https' : 'http') + '://apis.google.com/js/api.js',
        developerKey: 'AIzaSyAu4XVTngwlna-qCo330yC4YzmX7x4bIMA',
        clientId: '369311205870-44apjflpckc6eo7g4jvocu6rnr3uemb6.apps.googleusercontent.com',
        scope: ['https://www.googleapis.com/auth/drive']
    },
    {
        id: 'flickr',
        name: 'Flickr'
    },
    {
        id: 'one-drive',
        name: 'One Drive',
        libUrl: (isHttps ? 'https' : 'http') + '://js.live.net/v5.0/wl.js',
        clientId: '000000004812F65D',
        redirectUri: (isHttps ? 'https' : 'http') + '://pizaptest.pizap.com/wl-callback.html'
    },
    {
        id: 'picasa',
        name: 'Picasa'
    }
);
var currentSourceId = null;
var callback = null;
var loadedLibs = [];
var providers = {};
var multiselect = false;
var hideCancel = false;
var selectedProvider = null;
var proxyUrl = 'http://www.pizap.com/svc/image.php?url=';


// Helpers

// wait untill handler function return true
function waitFor(handler, next, timeout) {
    timeout = timeout || 100;
    var result = handler ? handler() : true;

    if (!result) {
        setTimeout(function () {
            waitFor(handler, next, timeout);
        }, timeout);
    } else {
        next && next();
    }
}

function loadLib(url, onloadCb, attrs) {
    attrs = attrs || {};

    if (loadedLibs.indexOf(url) !== -1 || url == null) {
        onloadCb && onloadCb();
        return;
    }

    var el = document.createElement('SCRIPT');
    el.setAttribute('src', url);

    if (onloadCb) {
        el.addEventListener('load', function () {
            onloadCb();
        }, false);
    }

    el.addEventListener('error', function () {
        loadedLibs.splice(loadedLibs.indexOf(this.src), 1);
    }, false);

    if (typeof attrs === 'object') {
        for (var k in attrs) {
            el.setAttribute(k, attrs[k]);
        }
    }

    var bodyEl = document.getElementsByTagName('BODY')[0];
    bodyEl.appendChild(el);

    loadedLibs.push(url);
}

function removeChildNodes(el) {
    while (el.firstChild) {
        el.removeChild(el.firstChild);
    }
}

function findParentNode(node, attrName) {
    if (!node) {
        return null;
    }

    while (node) {
        if (node.getAttribute && node.getAttribute(attrName) !== null) {
            break;
        }
        node = node.parentNode;
    }

    return node;
}

function activate(id) {
    var el = document.getElementById('filepicker-' + id);

    if (el) {
        el.classList.add('active');
    }
}

function deactivate(id) {
    var el = document.getElementById('filepicker-' + id);

    if (el) {
        el.classList.remove('active');
    }
}


// Functions

function callSource(id, cb) {
    var source = getSource(id);

    if (typeof providers[id] === 'undefined') {
        alert('Provider "' + id + '" is not found.');
        return;
    }

    source.multiselect = multiselect;

    currentSourceId = id;

    providers[id].call(null, source, callback);
    cb && cb();
}

function loadSourcesLibs() {
    for (var i = 0, len = sources.length; i < len; i++) {
        loadLib(sources[i].libUrl, (function (source) {
            return function () {
                typeof window['sourceLoaded-' + source.id] === 'function' && window['sourceLoaded-' + source.id]();
            };
        })(sources[i]), sources[i].attrs || null);
    }
}

function getSource(id) {
    var source = sources.filter(function (item) {
        return item.id == id;
    }).pop();

    if (!source) {
        alert('Source not found.');
        return null;
    }

    return source;
}

function renderSourceList(activeSources) {
    var listEl = document.getElementById('filepicker-sources-list');

    if (typeof activeSources !== 'undefined')
        activeSources.push("google");
    removeChildNodes(listEl);

    for (var i = 0, len = sources.length; i < len; i++) {
        if (typeof activeSources !== 'undefined' && activeSources.indexOf(sources[i].id) === -1) {
            continue;
        }

        listEl.insertAdjacentHTML('beforeend', sourceItemTemplate.render(sources[i]));
    }
}

function initSourcesList() {
    var listEl = document.getElementById('filepicker-sources-list');

    listEl.addEventListener('click', function (event) {
        var target = findParentNode(event.target, 'data-id');

        if (target !== null) {
            event.preventDefault();
            event.stopPropagation();

            deactivate('dialog');

            var sourceId = target.getAttribute('data-id');

            activate('dialog-loading');

            callSource(sourceId, function () {
                deactivate('dialog-loading');
            });
        }
    });

    sources.on('change', renderSourceList);
    renderSourceList();
}

function initDialog() {
    var html = dialogTemplate.render();
    var saveHtml = dialogSave.render();
    var finishHtml = dialogFinish.render();

    var bodyEl = document.getElementsByTagName('BODY')[0];

    bodyEl.insertAdjacentHTML('beforeend', html);
    bodyEl.insertAdjacentHTML('beforeend', saveHtml);
    bodyEl.insertAdjacentHTML('beforeend', finishHtml);


    var cancelBtn = document.getElementById('filepicker-cancel-btn');

    cancelBtn.addEventListener('click', function (event) {
        event.preventDefault();
        event.stopPropagation();

        deactivate('dialog');

        callback && callback(null);
    }, false);

    var saveCancelBtn = document.getElementById('save-cancel-btn');

    saveCancelBtn.addEventListener('click', function (event) {
        event.preventDefault();
        event.stopPropagation();
        var el = document.getElementById('save-dialog');

        if (el) {
            el.classList.remove('active');
        }

        callback && callback(null);
    }, false);

    var saveDoneBtn = document.getElementById('save-done-btn');

    saveDoneBtn.addEventListener('click', function (event) {
        event.preventDefault();
        event.stopPropagation();
        window.location.href='/pizap';
    }, false);

    var finishCancelBtn = document.getElementById('finish-cancel-btn');

    finishCancelBtn.addEventListener('click', function (event) {
        event.preventDefault();
        event.stopPropagation();
        var el = document.getElementById('finish-dialog');

        if (el) {
            el.classList.remove('active');
        }
    }, false);
}

function showDialog(activeSources) {
    renderSourceList(activeSources);
    activate('dialog');
    if (hideCancel) {
        var cancelBtn = document.getElementById('filepicker-cancel-btn');
        if (cancelBtn) {
            cancelBtn.style.display = "none";
        }
    } else {
        var cancelBtn = document.getElementById('filepicker-cancel-btn');
        if (cancelBtn) {
            cancelBtn.style.display = "block";
        }
    }
}


// Providers
providers['computer'] = require('./source-computer.js');
providers['facebook'] = require('./source-facebook.js');
providers['dropbox'] = require('./source-dropbox.js');
providers['one-drive'] = require('./source-one-drive.js');
providers['google-drive'] = require('./source-google-drive.js');
providers['flickr'] = require('./source-flickr.js');
providers['instagram'] = require('./source-instagram.js');
providers['picasa'] = require('./source-picasa.js');
providers['backgrounds'] = require('./source-backgrounds.js');
providers['webcam'] = require('./source-webcam.js');
providers['google'] = require('./source-google.js');
providers['border'] = require('./source-borders.js');

// Exports
var cloudUrl = null;
var cloudKey = null;
var uploadCallback = null;
var currentProgress;

function initSaveDialog() {
    var pizapSave = document.getElementById('pizapSave');
    var facebookSave = document.getElementById('facebookSave');
    var twitterSave = document.getElementById('twitterSave');
    var pinterestSave = document.getElementById('pinterestSave');
    var heartItSave = document.getElementById('heartItSave');
    var tumblrSave = document.getElementById('tumblrSave');
    var dropboxSave = document.getElementById('dropboxSave');
    var flickrSave = document.getElementById('flickrSave');
    var googleSave = document.getElementById('googleSave');

    var emailSave = document.getElementById('emailSave');
    var linkSave = document.getElementById('linkSave');
    var embedSave = document.getElementById('embedSave');
    var emailSubmit = document.getElementById('emailSubmit');

    var hidef = document.getElementById('hidef');
    var standard = document.getElementById('standard');
    var saveCaption = document.getElementById('saveCaption');

    if(saveCaption){
        saveCaption.addEventListener('blur', function(){
            var caption = document.getElementById('saveCaption').value;
            if (!caption || caption == "") {
            }else{
                ajaxJSONGet('/image/post?publicvalue=no&posttofacebook=no&imageNamePreset=' + cloudKey + '&title=' + encodeURIComponent(caption) + '&posttotwitter=no&posttotumblr=no');
            }
        });
    }

    if (pizapSave) {
        pizapSave.addEventListener('click', function _func(event) {
            currentProgress = document.getElementById('savePizapProgress');
            if (username == "") {
                PizapLogin.showLoginDialog();
                return false;
            }
            if (cloudUrl) {
                sharePizap();
            } else {
                var saveImg = document.getElementById("saveImg");
                if (saveImg) {
                    uploadCallback = sharePizap;
                    uploadFile(saveImg.src);
                }
            }
        });
    }

    if (facebookSave) {
        facebookSave.addEventListener('click', function _func(event) {
            currentProgress = document.getElementById('saveFacebookProgress');
           shareFacebook();
        });
    }

    if (twitterSave) {
        twitterSave.addEventListener('click', function _func(event) {
            currentProgress = document.getElementById('saveTwitterProgress');
            if (cloudUrl) {
                shareTwitter();
            } else {
                var saveImg = document.getElementById("saveImg");
                if (saveImg) {
                    uploadCallback = shareTwitter;
                    uploadFile(saveImg.src);
                }
            }
        });
    }

    if (pinterestSave) {
        pinterestSave.addEventListener('click', function _func(event) {
            currentProgress = document.getElementById('savePinterestProgress');
            if (cloudUrl) {
                sharePinterest();
            } else {
                var saveImg = document.getElementById("saveImg");
                if (saveImg) {
                    uploadCallback = sharePinterest;
                    uploadFile(saveImg.src);
                }
            }
        });
    }

    if (heartItSave) {
        heartItSave.addEventListener('click', function _func(event) {
            currentProgress = document.getElementById('saveWeHeartItProgress');
            if (cloudUrl) {
                shareHeartIt();
            } else {
                var saveImg = document.getElementById("saveImg");
                if (saveImg) {
                    uploadCallback = shareHeartIt;
                    uploadFile(saveImg.src);
                }
            }
        });
    }

    if (tumblrSave) {
        tumblrSave.addEventListener('click', function _func(event) {
            currentProgress = document.getElementById('saveTumblrProgress');
            if (cloudUrl) {
                shareTumblr();
            } else {
                var saveImg = document.getElementById("saveImg");
                if (saveImg) {
                    uploadCallback = shareTumblr;
                    uploadFile(saveImg.src);
                }
            }
        });
    }
    if (dropboxSave) {
        dropboxSave.addEventListener('click', function _func(event) {
            currentProgress = document.getElementById('saveDropboxProgress');
            if (cloudUrl) {
                shareDropbox();
            } else {
                var saveImg = document.getElementById("saveImg");
                if (saveImg) {
                    uploadCallback = shareDropbox;
                    uploadFile(saveImg.src);
                }
            }
        });
    }
    if (flickrSave) {
        flickrSave.addEventListener('click', function _func(event) {
            currentProgress = document.getElementById('saveFlickrProgress');
            if (cloudUrl) {
                shareFlickr();
            } else {
                var saveImg = document.getElementById("saveImg");
                if (saveImg) {
                    uploadCallback = shareFlickr;
                    uploadFile(saveImg.src);
                }
            }
        });
    }
    if (googleSave) {
        googleSave.addEventListener('click', function _func(event) {
            currentProgress = document.getElementById('saveGoogleProgress');
            var saveImg = document.getElementById("saveImg");
            if (saveImg) {
                logEvent("googleHtml");
                loadLib('http://apis.google.com/js/api.js', googleClientLoaded);
            }

        });
    }
    if (hidef) {
        hidef.addEventListener('change', function (event) {
            window.showStore();
            standard.checked = true;
            return false;
        });
    }

    if(embedSave){
        embedSave.addEventListener('click', function (event){
            var sel = document.getElementById("divEmbed");
            var pos = embedSave;
            if (!isVisible(sel)) {
                sel.style.top = findPos(pos)[1]  + "px";
                sel.style.left = (findPos(pos)[0] - 415) + "px";
                sel.style.display = "block";
            } else {
                sel.style.display = "none";
            }
            var taEmbed = document.getElementById('taEmbed');
            taEmbed.value = "<a href='http://www.pizap.com/image/" + cloudKey + "' target='_blank'><img src='" + cloudUrl + "' border='0' width='800'/><br></a><a style='font-size:12px' href='http://www.pizap.com/u/" + username + "' target='_blank'>" + username + "</a> <span style='font-size:12px'> on </span><a style='font-size:12px' href='http://www.pizap.com' target='_blank'>piZap.com</a>"
            return false;
        });
    }
    if(emailSave){
        emailSave.addEventListener('click', function (event){
            var sel = document.getElementById("divEmail");
            var pos = emailSave;
            if (!isVisible(sel)) {
                sel.style.top = findPos(pos)[1]  + "px";
                sel.style.left = (findPos(pos)[0] - 355) + "px";
                sel.style.display = "block";
            } else {
                sel.style.display = "none";
            }
            return false;
        });
    }
    if(linkSave){
        linkSave.addEventListener('click', function (event){
            var sel = document.getElementById("divLink");
            var pos = linkSave;
            if (!isVisible(sel)) {
                sel.style.top = findPos(pos)[1]  + "px";
                sel.style.left = (findPos(pos)[0] - 415) + "px";
                sel.style.display = "block";
            } else {
                sel.style.display = "none";
            }
            var taLink = document.getElementById('taLink');
            taLink.value = "http://www.pizap.com/image/" + cloudKey;
            return false;
        });
    }
    if(emailSubmit){
        emailSubmit.addEventListener('click', function (event){
            sendEmail(cloudUrl);
        });
    }
    window.twttr = (function (d, s, id) {
        var js, fjs = d.getElementsByTagName(s)[0],
            t = window.twttr || {};
        if (d.getElementById(id)) return t;
        js = d.createElement(s);
        js.id = id;
        js.src = "https://platform.twitter.com/widgets.js";
        fjs.parentNode.insertBefore(js, fjs);

        t._e = [];
        t.ready = function (f) {
            t._e.push(f);
        };

        return t;
    }(document, "script", "twitter-wjs"));
}

function googleClientLoaded() {
    document.getElementById('saveGoogleProgress').value = .25;
    var source = getSource('google-drive');

    window.gapi.auth.authorize(
        {
            client_id: source.clientId,
            scope: source.scope,
            immediate: false
        },
        handleAuthResult
    );

}
var driveOauthToken;
function handleAuthResult(authResult) {
    document.getElementById('saveGoogleProgress').value = 1;

    if (authResult && !authResult.error) {
        driveOauthToken = authResult.access_token;
        var source = getSource('google-drive');
        window.gapi.client.setApiKey(source.developerKey);
        googleDriveSave(saveImg.src, function (response) {
            document.getElementById('saveGoogleCheck').style.display = '';
            document.getElementById('saveGoogleProgress').value = 1;

            //alert("Image saved to Google Drive");
        });
    }
}

function sharePizap() {
    document.getElementById('savePizapCheck').style.display = '';
    document.getElementById('savePizapProgress').value = 1;
    logEvent("pizapHtml");
    var caption = document.getElementById('saveCaption').value;
    if (!caption || caption == "") {
        caption = "";
    }
    ajaxJSONGet('/image/post?publicvalue=yes&posttofacebook=no&imageNamePreset=' + cloudKey + '&title=' + encodeURIComponent(caption) + '&posttotwitter=no&posttotumblr=no');

    //alert("Image save to piZap");
}
function shareTumblr() {
    document.getElementById('saveTumblrCheck').style.display = '';
    document.getElementById('saveTumblrProgress').value = 1;
    var encodedUrl = encodeURIComponent('http://' + baseUrl + '/image/' + cloudKey);
    logEvent("tumblrHtml");
    window.open('http://www.tumblr.com/share/photo?click_thru=' + encodedUrl + '&source=' + encodeURIComponent(cloudUrl), '', 'menubar=no,toolbar=no,resizable=yes,scrollbars=yes,height=600,width=600');
    return false;
}

function shareHeartIt() {
    document.getElementById('saveWeHeartItCheck').style.display = '';
    document.getElementById('saveWeHeartItProgress').value = 1;
    var encodedUrl = encodeURIComponent('http://' + baseUrl + '/image/' + cloudKey);
    logEvent("weHeartItHtml");
    window.open('http://weheartit.com/heart-it/heart?source_url=' + encodedUrl + '&image_url=' + encodeURIComponent(cloudUrl), '', 'menubar=no,toolbar=no,resizable=yes,scrollbars=yes,height=600,width=600');
    return false;
}

function sharePinterest() {
    document.getElementById('savePinterestCheck').style.display = '';
    document.getElementById('savePinterestProgress').value = 1;
    var encodedUrl = encodeURIComponent('http://' + baseUrl + '/image/' + cloudKey);
    logEvent("pinterestHtml");
    window.open('https://pinterest.com/pin/create/button/?url=' + encodedUrl + '&media=' + encodeURIComponent(cloudUrl), '', 'menubar=no,toolbar=no,resizable=yes,scrollbars=yes,height=600,width=600');
    return false;
}

function shareTwitter() {
    document.getElementById('saveTwitterCheck').style.display = '';
    document.getElementById('saveTwitterProgress').value = 1;
    var encodedUrl = encodeURIComponent('http://' + baseUrl + '/image/' + cloudKey);
    logEvent("twitterHtml");
    window.open('https://twitter.com/share?url=' + encodedUrl + '&via=pizap', '', 'menubar=no,toolbar=no,resizable=yes,scrollbars=yes,height=600,width=600');
    return false;
}

function shareDropbox() {
    var at = {
        'id': 'dropboxjs',
        'data-app-key': 'krklwcfzbe1m0f5'
    }
    logEvent("dropboxHtml");
    loadLib('https://www.dropbox.com/static/api/2/dropins.js', dropboxLoaded, at);
}
function shareFlickr() {
    document.getElementById('saveFlickrProgress').value = 1;
    logEvent("flickrHtml");
    window.open("/flickr/write", '', 'width=800,height=600');
}

function flickrUpload(response) {
    document.getElementById('saveFlickrCheck').style.display = '';
    document.getElementById('saveFlickrProgress').value = 1;
    if (response && response.stat == "ok") {
        //alert("Flickr image uploaded");
    }
    else {
        alert("Flickr image FAILED to upload");
    }
}

function dropboxLoaded() {
    document.getElementById('saveDropboxCheck').style.display = '';
    document.getElementById('saveDropboxProgress').value = 1;

    var options = {
        success: function () {
            // Indicate to the user that the files have been saved.
            //alert("Success! Files saved to your Dropbox.");
        },
        progress: function (progress) {
            document.getElementById('saveDropboxProgress').value = progress;
        },
        cancel: function () {
        },
        error: function (errorMessage) {
        }
    };
    Dropbox.save(cloudUrl, cloudKey, options);
    return false;

}

function doesUserHavePublishActions() {
    FB.api('/me/permissions', function (response) {
        if (response && response.data) {
            var found = false;
            for (var x = 0; x < response.data.length; x++) {
                if (response.data[x].permission == 'publish_actions' && response.data[x].status == "granted") {
                    found = true;
                }
            }
            if (found) {
                var saveImg = document.getElementById("saveImg");
                var blob = dataURLtoBlob(saveImg.src);

                var authResponse = FB.getAuthResponse();

                var fd = new FormData();
                cloudKey = "pizap.jpg";

                fd.append("source", blob, cloudKey);
                var xhr = new XMLHttpRequest();

                xhr.upload.addEventListener("progress", function (evt) {
                    if (evt.lengthComputable) {
                        document.getElementById('saveFacebookProgress').value = (evt.loaded / evt.total) / 2;
                    }
                }, false);
                xhr.addEventListener("load", function (evt) {
                    var xmlDoc = evt.target.responseXML;
                    document.getElementById('saveFacebookProgress').value = 100;
                }, false);
                xhr.addEventListener("error", function (evt) {
                    console.log("upload Failed");
                }, false);
                xhr.addEventListener("abort", function (evt) {
                    console.log("upload cancelled");
                }, false);

                xhr.open('POST', 'https://graph.facebook.com/me/photos?access_token=' + authResponse.accessToken, true);

                xhr.send(fd);
            } else {
                FB.login(function (response) {
                    if (response.status === 'connected') {
                        doesUserHavePublishActions();
                    }
                }, {scope: 'email, publish_actions'});
            }
        } else {
            FB.login(function (response) {
                if (response.status === 'connected') {
                    doesUserHavePublishActions();
                }
            }, {scope: 'email, publish_actions'});
        }
    });
}

function shareFacebook() {
    logEvent("facebookHtml");
    FB.getLoginStatus(function (response) {
        if (response.status === 'connected') {
            doesUserHavePublishActions();
        } else if (response.status === 'not_authorized') {
            FB.login(function (response) {
                if (response.status === 'connected') {
                    doesUserHavePublishActions();
                }
            }, {scope: 'email, publish_actions'});
        } else {
            FB.login(function (response) {
                if (response.status === 'connected') {
                    doesUserHavePublishActions();
                }
            }, {scope: 'email, publish_actions'});
        }
    });


}

function ajaxJSONGet(url, callback) {
    var http_request = new XMLHttpRequest();
    http_request.open("GET", url, true);
    http_request.onreadystatechange = function () {
        var done = 4;
        var ok = 200;
        if (http_request.readyState === done && http_request.status === ok) {
            if (callback)
                callback(JSON.parse(http_request.responseText));
        }
    };
    http_request.send();
}
function makeid() {
    var text = "";
    var possible = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";

    for (var i = 0; i < 5; i++)
        text += possible.charAt(Math.floor(Math.random() * possible.length));

    return text;
}
function uploadFile(dataURI) {
    ajaxJSONGet('/image/post', function (response) {
        var blob = dataURLtoBlob(dataURI);
        var fd = new FormData();

        cloudKey = response.ImagePrefix + ".jpg";

        fd.append("x-amz-storage-class", "REDUCED_REDUNDANCY");
        fd.append('policy', response.Policy)
        fd.append('signature', response.Signature);
        fd.append('acl', 'public-read');
        fd.append('key', cloudKey);
        fd.append('AWSAccessKeyId', response.AccessKey);
        fd.append('Content-Type', "image/jpeg");
        fd.append("success_action_status", "201");
        fd.append("file", blob, cloudKey);
        var xhr = new XMLHttpRequest();

        xhr.upload.addEventListener("progress", uploadProgress, false);
        xhr.addEventListener("load", uploadComplete, false);
        xhr.addEventListener("error", uploadFailed, false);
        xhr.addEventListener("abort", uploadCanceled, false);

        xhr.open('POST', '//s3-us-west-1.amazonaws.com/imagesw.pizap.com/', true); //MUST BE LAST LINE BEFORE YOU SEND

        xhr.send(fd);
    });
}

function dataURLtoBlob(dataurl) {
    var arr = dataurl.split(','), mime = arr[0].match(/:(.*?);/)[1],
        bstr = atob(arr[1]), n = bstr.length, u8arr = new Uint8Array(n);
    while (n--) {
        u8arr[n] = bstr.charCodeAt(n);
    }
    return new Blob([u8arr], {type: mime});
}
function uploadProgress(evt) {
    if (evt.lengthComputable) {
        var percentComplete = (evt.loaded / evt.total) / 2;
        if (currentProgress) currentProgress.value = percentComplete;
    }
}

function uploadComplete(evt) {
    var xmlDoc = evt.target.responseXML;
    cloudUrl = xmlDoc.getElementsByTagName("Location")[0].textContent;
    console.log("S3 file location = " + cloudUrl);
    var caption = document.getElementById('saveCaption').value;
    if (!caption || caption == "") {
        caption = "";
    }
    ajaxJSONGet('/image/post?publicvalue=no&posttofacebook=no&imageNamePreset=' + cloudKey + '&title=' + encodeURIComponent(caption) + '&posttotwitter=no&posttotumblr=no', uploadCallback);
    var dests = document.getElementById("saveDestinations");
    if(dests){
        dests.style.display = "block";
    }

    var finishButtons = document.getElementById("finishButtons");
    if(finishButtons){
        finishButtons.style.display = "block";
    }

    var finishSaveToPizap = document.getElementById("finishSaveToPizap");
    if(finishSaveToPizap){
        finishSaveToPizap.style.display = "none";
    }

}

function uploadFailed(evt) {
    alert("There was an error attempting to upload the file." + evt);
}

function uploadCanceled(evt) {
    alert("The upload has been canceled by the user or the browser dropped the connection.");
}

/**
 * Insert new file.
 *
 * @param {File} fileData File object to read data from.
 * @param {Function} callback Function to call when the request is complete.
 */
function googleDriveSave(fileData, callback) {
    var boundary = '-------314159265358979323846';
    var delimiter = "\r\n--" + boundary + "\r\n";
    var close_delim = "\r\n--" + boundary + "--";

    var arr = fileData.split(','), mime = arr[0].match(/:(.*?);/)[1];

    var metadata = {
        'title': makeid() + 'pizapw' + (Date.now() / 1000 | 0),
        'mimeType': mime
    };

    var base64Data = btoa(arr[1]);
    var multipartRequestBody =
        delimiter +
        'Content-Type: application/json\r\n\r\n' +
        JSON.stringify(metadata) +
        delimiter +
        'Content-Type: ' + mime + '\r\n' +
        'Content-Transfer-Encoding: base64\r\n' +
        '\r\n' +
        arr[1] +
        close_delim;

    var request = gapi.client.request({
        'path': '/upload/drive/v2/files',
        'method': 'POST',
        'params': {'uploadType': 'multipart'},
        'headers': {
            'Content-Type': 'multipart/mixed; boundary="' + boundary + '"'
        },
        'body': multipartRequestBody
    });
    if (!callback) {
        callback = function (file) {
            console.log(file)
        };
    }

    request.execute(callback);
}


window.filepicker = {
    pick: function (cb, activeSource, multiselectEnabled, firstLoad, showProvider) {
        if (typeof multiselectEnabled !== 'undefined') {
            multiselect = !!multiselectEnabled;
        }
        if (typeof firstLoad !== 'undefined') {
            hideCancel = !!firstLoad;
        }
        if (typeof showProvider !== 'undefined') {
            selectedProvider = !!showProvider;
        }

        callback = function (result) {
            if (typeof cb === 'function') {
                cb && cb.apply(null, [result]);
            } else if (typeof cb === 'string') {
                try {
                    if (result === null) {
                        result = [];
                    }

                    var flashMovie = getFlashMovieObject('flashContent');

                    flashMovie && flashMovie.filePicked(result);

                    // flashMovie && Function.prototype.apply(flashMovie[cb], [null, arguments]);
                } catch (e) {
                    alert('Can\'t call callback "' + cb + '" from "flashContent" moovie. Error: ' + e);
                }
                // window[cb].apply(null, arguments);
            } else {
                alert('You doesn\'t set callback for File Picker.');
            }
        };

        activate('dialog-loading');

        waitFor(function () {
            return typeof window.Dropbox !== 'undefined' && typeof window.gapi !== 'undefined' && typeof window.WL !== 'undefined';
        }, function () {
            deactivate('dialog-loading');
        });

        if (typeof activeSource === 'string') {
            activate('dialog-loading');

            callSource(activeSource, function () {
                deactivate('dialog-loading');
            });
        } else if (typeof activeSource !== 'undefined' && activeSource instanceof Array) {
            showDialog(activeSource);
        } else {
            showDialog();
        }
    }, setInstagramToken: function (token, username) {
        var id = 'instagram';
        var source = getSource(id);
        source.username = username;
        source.token = token;
        callSource(id);
        console.log("instagram token received: " + token);
    }, setFlickrUserName: function (flickrUserName) {
        var id = 'flickr';
        var source = getSource(id);
        source.username = flickrUserName;
        callSource(id);

    }, setPicasaToken: function () {
        var id = 'picasa';
        var source = getSource(id);
        source.token = true;
        callSource(id);
    }

};

window.saveDialog = {
    show: function (imageData) {
        cloudUrl = null;
        cloudKey = null;
        uploadCallback = null;
        currentProgress = document.getElementById("saveMainProgress");
        document.getElementById('savePizapProgress').value = 0;
        document.getElementById('saveTumblrProgress').value = 0;
        document.getElementById('saveWeHeartItProgress').value = 0;
        document.getElementById('saveFacebookProgress').value = 0;
        document.getElementById('saveTwitterProgress').value = 0;
        document.getElementById('savePinterestProgress').value = 0;
        document.getElementById('saveDropboxProgress').value = 0;
        document.getElementById('saveFlickrProgress').value = 0;
        document.getElementById('saveGoogleProgress').value = 0;
        document.getElementById('saveEmailProgress').value = 0;
        document.getElementById('saveLinkProgress').value = 0;
        document.getElementById('saveEmbedProgress').value = 0;

        document.getElementById('savePizapCheck').style.display = 'none';
        document.getElementById('saveTumblrCheck').style.display = 'none';
        document.getElementById('saveWeHeartItCheck').style.display = 'none';
        document.getElementById('saveFacebookCheck').style.display = 'none';
        document.getElementById('saveTwitterCheck').style.display = 'none';
        document.getElementById('savePinterestCheck').style.display = 'none';
        document.getElementById('saveDropboxCheck').style.display = 'none';
        document.getElementById('saveFlickrCheck').style.display = 'none';
        document.getElementById('saveGoogleCheck').style.display = 'none';
        document.getElementById('saveEmailCheck').style.display = 'none';
        document.getElementById('saveLinkCheck').style.display = 'none';
        document.getElementById('saveEmbedCheck').style.display = 'none';

        var el = document.getElementById('save-dialog');
        if (el) {
            el.classList.add('active');
        }

        var saveImg = document.getElementById("saveImg");
        saveImg.onload = function () {
            //uploadFile(this.src);
        };
        saveImg.src = imageData;

        var standardText = document.getElementById('standardText');
        if (proPurchased && proPurchased == true) {
            standardText.style.display = 'none';
            standard.style.display = 'none';
            hidef.style.display = 'none';
        } else {
            standardText.style.display = '';
            standard.style.display = '';
            hidef.style.display = '';
        }
    }, setFlickrUserName: function (flickrUserName) {
        ajaxJSONGet('/flickr/upload?imageName=' + cloudKey, flickrUpload);
    }, showFinish: function() {
        var finishButtons = document.getElementById("finishButtons");
        if(finishButtons){
            finishButtons.style.display = "block";
        }
        var finishSaveToPizap = document.getElementById("finishSaveToPizap");
        if(finishSaveToPizap){
            finishSaveToPizap.style.display = "none";
        }

        var el = document.getElementById('finish-dialog');
        if (el) {
            el.classList.add('active');
        }
        if(proPurchased){
            var proButton = document.getElementById('upgradePro');
            if(proButton){
                proButton.style.display = 'none';
            }
        }
    }, savePizap: function(imageData) {
        var saveImg = document.getElementById("saveImg");
        saveImg.onload = function () {
            var finishButtons = document.getElementById("finishButtons");
            if(finishButtons){
                finishButtons.style.display = "none";
            }
            var finishSaveToPizap = document.getElementById("finishSaveToPizap");
            if(finishSaveToPizap){
                finishSaveToPizap.style.display = "block";
            }
            currentProgress = document.getElementById("saveToPizap");
            uploadFile(this.src);
        };
        saveImg.src = imageData;

    }

};
// Start

initDialog();
initSourcesList();
loadSourcesLibs();
initSaveDialog();

window.getMemeMenu = function(){
    var listEl = document.getElementById('menu-container');
    while (listEl.firstChild) {
        listEl.removeChild(listEl.firstChild);
    }
    listEl.insertAdjacentHTML('beforeend', memeTemplate.render({}));
    $('#memeTopText').bind('input', function(){
        var editor = getFlashMovieObject("Editor");
        editor.changeMemeText('TOP', $(this).val());
    });
    $('#memeBottomText').bind('input', function(){
        var editor = getFlashMovieObject("Editor");
        editor.changeMemeText('BOTTOM', $(this).val());
    });
    $('#memeTopPlus').click(function(){
        var editor = getFlashMovieObject("Editor");
        editor.memeIncreaseText('TOP');
    });
    $('#memeTopMinus').click(function(){
        var editor = getFlashMovieObject("Editor");
        editor.memeDecreaseText('TOP');
    });
    $('#memeBottomPlus').click(function(){
        var editor = getFlashMovieObject("Editor");
        editor.memeIncreaseText('BOTTOM');
    });
    $('#memeBottomMinus').click(function(){
        var editor = getFlashMovieObject("Editor");
        editor.memeDecreaseText('BOTTOM');
    });
    $("#previewTop").click(function(e) {
        var pickerCont = document.getElementById('colorpicker');
        pickerCont.style.display = "block";
        bCanPreview = true;
        window.pickerControl = this;
        window.memeLocation = "TOP";
    });
    $("#previewBottom").click(function(e) {
        var pickerCont = document.getElementById('colorpicker');
        pickerCont.style.display = "block";
        bCanPreview = true;
        window.pickerControl = this;
        window.memeLocation = "BOTTOM";
    });

}

window.getStickerFolders = function(){
    var listEl = document.getElementById('menu-container');
    while (listEl.firstChild) {
        listEl.removeChild(listEl.firstChild);
    }

    var listEl = document.getElementById('menu-container');
    while (listEl.firstChild) {
        listEl.removeChild(listEl.firstChild);
    }
    var basePath = assets.CDNPath + assets.Stickers.Path;
    if('https:' == document.location.protocol){
        basePath = assets.CDNSSLPath + assets.Stickers.Path;
    }
    var last = "";

    for(var i = 0; i < assets.Stickers.items.length;i++) {
        if(last != assets.Stickers.items[i].Name) {
            listEl.insertAdjacentHTML('beforeend', assetFolderTemplate.render({
                id: assets.Stickers.items[i].Name,
                name: assets.Stickers.items[i].Name,
                icon: basePath + assets.Stickers.items[i].Thumb
            }));
        }
        last = assets.Stickers.items[i].Name;
    }
}
window.getStickerFolder = function(name){
    var listEl = document.getElementById('menu-container');
    while (listEl.firstChild) {
        listEl.removeChild(listEl.firstChild);
    }

    var listEl = document.getElementById('menu-container');
    while (listEl.firstChild) {
        listEl.removeChild(listEl.firstChild);
    }
    var basePath = assets.CDNPath + assets.Stickers.Path;
    if('https:' == document.location.protocol){
        basePath = assets.CDNSSLPath + assets.Stickers.Path;
    }

    for(var i = 0; i < assets.Stickers.items.length;i++) {
        if(name == assets.Stickers.items[i].Name) {
            var item = assets.Stickers.items[i];
            listEl.insertAdjacentHTML('beforeend', '<div class="back"><span>&lt; Back</span>');
            $('.back').click(function (event){
                resetMenus(this);
                $('#menu-container').html("Stickers");
                getStickerFolders();
                currentMenu = "sticker";
            });

            addStickers(item, basePath);
            if(item.Pages){
                for(var j = 0; j < item.Pages.length;j++) {
                    addStickers(item.Pages[j], basePath);
                }
            }
        }
    }
}
function addStickers(item, path){
    var listEl = document.getElementById('menu-container');
    var basePath = path + item.Path;
    var thumbnailExt = "png";
    var thumbnailPrefix = "thumb";
    var thumbnailDigits = 1;
    var pathPrefix = "img";
    var pathExt = "png";
    var pathDigits = 1;

    if(item.ThumbnailExt) thumbnailExt = item.ThumbnailExt;
    if(item.ThumbnailPrefix) thumbnailPrefix = item.ThumbnailPrefix;
    if(item.ThumbnailDigits) thumbnailDigits = item.ThumbnailDigits;

    if(item.PathExt) pathExt = item.PathExt;
    if(item.PathPrefix) pathPrefix = item.PathPrefix;
    if(item.PathDigits) pathDigits = item.PathDigits;

    for(var j = 1; j <= item.ItemCount;j++){
        var thumb = basePath + thumbnailPrefix + zeroPad(j, thumbnailDigits) + "." + thumbnailExt;
        var source = basePath + pathPrefix + zeroPad(j, pathDigits) + "." + pathExt;

        var p = {id: name + j, source: source, picture: thumb};
        listEl.insertAdjacentHTML('beforeend', photoItemTemplate.render(p));
    }

}
function zeroPad(num, places) {
    var zero = places - num.toString().length + 1;
    return Array(+(zero > 0 && zero)).join("0") + num;
}

window.getBorderFolders = function(){
    var listEl = document.getElementById('menu-container');
    while (listEl.firstChild) {
        listEl.removeChild(listEl.firstChild);
    }

    var listEl = document.getElementById('menu-container');
    while (listEl.firstChild) {
        listEl.removeChild(listEl.firstChild);
    }
    var basePath = assets.CDNPath + assets.Borders.Path;
    if('https:' == document.location.protocol){
        basePath = assets.CDNSSLPath + assets.Borders.Path;
    }
    var last = "";

    for(var i = 0; i < assets.Borders.items.length;i++) {
        if(last != assets.Borders.items[i].Name) {
            listEl.insertAdjacentHTML('beforeend', assetFolderTemplate.render({
                id: assets.Borders.items[i].Name,
                name: assets.Borders.items[i].Name,
                icon: basePath + assets.Borders.items[i].Thumb
            }));
        }
        last = assets.Borders.items[i].Name;
    }
}

window.getBorderFolder = function(name){
    var listEl = document.getElementById('menu-container');
    while (listEl.firstChild) {
        listEl.removeChild(listEl.firstChild);
    }

    var listEl = document.getElementById('menu-container');
    while (listEl.firstChild) {
        listEl.removeChild(listEl.firstChild);
    }
    var basePath = assets.CDNPath + assets.Borders.Path;
    if('https:' == document.location.protocol){
        basePath = assets.CDNSSLPath + assets.Borders.Path;
    }

    for(var i = 0; i < assets.Borders.items.length;i++) {
        if(name == assets.Borders.items[i].Name) {
            var item = assets.Borders.items[i];
            addStickers(item, basePath);
            if(item.Pages){
                for(var j = 0; j < item.Pages.length;j++) {
                    addStickers(item.Pages[j], basePath);
                }
            }
        }
    }
}
function downloadFile(downloadUrl, callback) {
    var xhr = new XMLHttpRequest();
    xhr.open('GET', downloadUrl);
    if ( xhr.overrideMimeType ) xhr.overrideMimeType("text/plain; charset=x-user-defined");
    xhr.onload = function() {
        callback(xhr.responseText);
    };
    xhr.onerror = function() {
        callback(null);
    };
    xhr.send();
}

function ajaxJSONGet(url, callback){
    var http_request = new XMLHttpRequest();
    http_request.open("GET", url, true);
    http_request.onreadystatechange = function () {
        var done = 4;
        var ok = 200;
        if (http_request.readyState === done && http_request.status === ok && callback){
            callback(JSON.parse(http_request.responseText));
        }
    };
    http_request.send();
}

}, {"visionmedia/superagent@1.1.0":2,"jmas/arr@0.1.4":3,"./templates/dialog.hogan":4,"./templates/source-item.hogan":5,"./templates/save-dialog.hogan":6,"./templates/finish-dialog.hogan":7,"./templates/asset-folder.hogan":8,"./templates/photo-item.hogan":9,"./templates/meme.hogan":10,"./source-computer.js":11,"./source-facebook.js":12,"./source-dropbox.js":13,"./source-one-drive.js":14,"./source-google-drive.js":15,"./source-flickr.js":16,"./source-instagram.js":17,"./source-picasa.js":18,"./source-backgrounds.js":19,"./source-webcam.js":20,"./source-google.js":21,"./source-borders.js":22}],
2: [function(require, module, exports) {
/**
 * Module dependencies.
 */

var Emitter = require('emitter');
var reduce = require('reduce');

/**
 * Root reference for iframes.
 */

var root = 'undefined' == typeof window
  ? this
  : window;

/**
 * Noop.
 */

function noop(){};

/**
 * Check if `obj` is a host object,
 * we don't want to serialize these :)
 *
 * TODO: future proof, move to compoent land
 *
 * @param {Object} obj
 * @return {Boolean}
 * @api private
 */

function isHost(obj) {
  var str = {}.toString.call(obj);

  switch (str) {
    case '[object File]':
    case '[object Blob]':
    case '[object FormData]':
      return true;
    default:
      return false;
  }
}

/**
 * Determine XHR.
 */

request.getXHR = function () {
  if (root.XMLHttpRequest
    && ('file:' != root.location.protocol || !root.ActiveXObject)) {
    return new XMLHttpRequest;
  } else {
    try { return new ActiveXObject('Microsoft.XMLHTTP'); } catch(e) {}
    try { return new ActiveXObject('Msxml2.XMLHTTP.6.0'); } catch(e) {}
    try { return new ActiveXObject('Msxml2.XMLHTTP.3.0'); } catch(e) {}
    try { return new ActiveXObject('Msxml2.XMLHTTP'); } catch(e) {}
  }
  return false;
};

/**
 * Removes leading and trailing whitespace, added to support IE.
 *
 * @param {String} s
 * @return {String}
 * @api private
 */

var trim = ''.trim
  ? function(s) { return s.trim(); }
  : function(s) { return s.replace(/(^\s*|\s*$)/g, ''); };

/**
 * Check if `obj` is an object.
 *
 * @param {Object} obj
 * @return {Boolean}
 * @api private
 */

function isObject(obj) {
  return obj === Object(obj);
}

/**
 * Serialize the given `obj`.
 *
 * @param {Object} obj
 * @return {String}
 * @api private
 */

function serialize(obj) {
  if (!isObject(obj)) return obj;
  var pairs = [];
  for (var key in obj) {
    if (null != obj[key]) {
      pairs.push(encodeURIComponent(key)
        + '=' + encodeURIComponent(obj[key]));
    }
  }
  return pairs.join('&');
}

/**
 * Expose serialization method.
 */

 request.serializeObject = serialize;

 /**
  * Parse the given x-www-form-urlencoded `str`.
  *
  * @param {String} str
  * @return {Object}
  * @api private
  */

function parseString(str) {
  var obj = {};
  var pairs = str.split('&');
  var parts;
  var pair;

  for (var i = 0, len = pairs.length; i < len; ++i) {
    pair = pairs[i];
    parts = pair.split('=');
    obj[decodeURIComponent(parts[0])] = decodeURIComponent(parts[1]);
  }

  return obj;
}

/**
 * Expose parser.
 */

request.parseString = parseString;

/**
 * Default MIME type map.
 *
 *     superagent.types.xml = 'application/xml';
 *
 */

request.types = {
  html: 'text/html',
  json: 'application/json',
  xml: 'application/xml',
  urlencoded: 'application/x-www-form-urlencoded',
  'form': 'application/x-www-form-urlencoded',
  'form-data': 'application/x-www-form-urlencoded'
};

/**
 * Default serialization map.
 *
 *     superagent.serialize['application/xml'] = function(obj){
 *       return 'generated xml here';
 *     };
 *
 */

 request.serialize = {
   'application/x-www-form-urlencoded': serialize,
   'application/json': JSON.stringify
 };

 /**
  * Default parsers.
  *
  *     superagent.parse['application/xml'] = function(str){
  *       return { object parsed from str };
  *     };
  *
  */

request.parse = {
  'application/x-www-form-urlencoded': parseString,
  'application/json': JSON.parse
};

/**
 * Parse the given header `str` into
 * an object containing the mapped fields.
 *
 * @param {String} str
 * @return {Object}
 * @api private
 */

function parseHeader(str) {
  var lines = str.split(/\r?\n/);
  var fields = {};
  var index;
  var line;
  var field;
  var val;

  lines.pop(); // trailing CRLF

  for (var i = 0, len = lines.length; i < len; ++i) {
    line = lines[i];
    index = line.indexOf(':');
    field = line.slice(0, index).toLowerCase();
    val = trim(line.slice(index + 1));
    fields[field] = val;
  }

  return fields;
}

/**
 * Return the mime type for the given `str`.
 *
 * @param {String} str
 * @return {String}
 * @api private
 */

function type(str){
  return str.split(/ *; */).shift();
};

/**
 * Return header field parameters.
 *
 * @param {String} str
 * @return {Object}
 * @api private
 */

function params(str){
  return reduce(str.split(/ *; */), function(obj, str){
    var parts = str.split(/ *= */)
      , key = parts.shift()
      , val = parts.shift();

    if (key && val) obj[key] = val;
    return obj;
  }, {});
};

/**
 * Initialize a new `Response` with the given `xhr`.
 *
 *  - set flags (.ok, .error, etc)
 *  - parse header
 *
 * Examples:
 *
 *  Aliasing `superagent` as `request` is nice:
 *
 *      request = superagent;
 *
 *  We can use the promise-like API, or pass callbacks:
 *
 *      request.get('/').end(function(res){});
 *      request.get('/', function(res){});
 *
 *  Sending data can be chained:
 *
 *      request
 *        .post('/user')
 *        .send({ name: 'tj' })
 *        .end(function(res){});
 *
 *  Or passed to `.send()`:
 *
 *      request
 *        .post('/user')
 *        .send({ name: 'tj' }, function(res){});
 *
 *  Or passed to `.post()`:
 *
 *      request
 *        .post('/user', { name: 'tj' })
 *        .end(function(res){});
 *
 * Or further reduced to a single call for simple cases:
 *
 *      request
 *        .post('/user', { name: 'tj' }, function(res){});
 *
 * @param {XMLHTTPRequest} xhr
 * @param {Object} options
 * @api private
 */

function Response(req, options) {
  options = options || {};
  this.req = req;
  this.xhr = this.req.xhr;
  // responseText is accessible only if responseType is '' or 'text' and on older browsers
  this.text = ((this.req.method !='HEAD' && (this.xhr.responseType === '' || this.xhr.responseType === 'text')) || typeof this.xhr.responseType === 'undefined')
     ? this.xhr.responseText
     : null;
  this.statusText = this.req.xhr.statusText;
  this.setStatusProperties(this.xhr.status);
  this.header = this.headers = parseHeader(this.xhr.getAllResponseHeaders());
  // getAllResponseHeaders sometimes falsely returns "" for CORS requests, but
  // getResponseHeader still works. so we get content-type even if getting
  // other headers fails.
  this.header['content-type'] = this.xhr.getResponseHeader('content-type');
  this.setHeaderProperties(this.header);
  this.body = this.req.method != 'HEAD'
    ? this.parseBody(this.text ? this.text : this.xhr.response)
    : null;
}

/**
 * Get case-insensitive `field` value.
 *
 * @param {String} field
 * @return {String}
 * @api public
 */

Response.prototype.get = function(field){
  return this.header[field.toLowerCase()];
};

/**
 * Set header related properties:
 *
 *   - `.type` the content type without params
 *
 * A response of "Content-Type: text/plain; charset=utf-8"
 * will provide you with a `.type` of "text/plain".
 *
 * @param {Object} header
 * @api private
 */

Response.prototype.setHeaderProperties = function(header){
  // content-type
  var ct = this.header['content-type'] || '';
  this.type = type(ct);

  // params
  var obj = params(ct);
  for (var key in obj) this[key] = obj[key];
};

/**
 * Parse the given body `str`.
 *
 * Used for auto-parsing of bodies. Parsers
 * are defined on the `superagent.parse` object.
 *
 * @param {String} str
 * @return {Mixed}
 * @api private
 */

Response.prototype.parseBody = function(str){
  var parse = request.parse[this.type];
  return parse && str && (str.length || str instanceof Object)
    ? parse(str)
    : null;
};

/**
 * Set flags such as `.ok` based on `status`.
 *
 * For example a 2xx response will give you a `.ok` of __true__
 * whereas 5xx will be __false__ and `.error` will be __true__. The
 * `.clientError` and `.serverError` are also available to be more
 * specific, and `.statusType` is the class of error ranging from 1..5
 * sometimes useful for mapping respond colors etc.
 *
 * "sugar" properties are also defined for common cases. Currently providing:
 *
 *   - .noContent
 *   - .badRequest
 *   - .unauthorized
 *   - .notAcceptable
 *   - .notFound
 *
 * @param {Number} status
 * @api private
 */

Response.prototype.setStatusProperties = function(status){
  var type = status / 100 | 0;

  // status / class
  this.status = status;
  this.statusType = type;

  // basics
  this.info = 1 == type;
  this.ok = 2 == type;
  this.clientError = 4 == type;
  this.serverError = 5 == type;
  this.error = (4 == type || 5 == type)
    ? this.toError()
    : false;

  // sugar
  this.accepted = 202 == status;
  this.noContent = 204 == status || 1223 == status;
  this.badRequest = 400 == status;
  this.unauthorized = 401 == status;
  this.notAcceptable = 406 == status;
  this.notFound = 404 == status;
  this.forbidden = 403 == status;
};

/**
 * Return an `Error` representative of this response.
 *
 * @return {Error}
 * @api public
 */

Response.prototype.toError = function(){
  var req = this.req;
  var method = req.method;
  var url = req.url;

  var msg = 'cannot ' + method + ' ' + url + ' (' + this.status + ')';
  var err = new Error(msg);
  err.status = this.status;
  err.method = method;
  err.url = url;

  return err;
};

/**
 * Expose `Response`.
 */

request.Response = Response;

/**
 * Initialize a new `Request` with the given `method` and `url`.
 *
 * @param {String} method
 * @param {String} url
 * @api public
 */

function Request(method, url) {
  var self = this;
  Emitter.call(this);
  this._query = this._query || [];
  this.method = method;
  this.url = url;
  this.header = {};
  this._header = {};
  this.on('end', function(){
    var err = null;
    var res = null;

    try {
      res = new Response(self);
    } catch(e) {
      err = new Error('Parser is unable to parse the response');
      err.parse = true;
      err.original = e;
      return self.callback(err);
    }

    self.emit('response', res);

    if (err) {
      return self.callback(err, res);
    }

    if (res.status >= 200 && res.status < 300) {
      return self.callback(err, res);
    }

    var new_err = new Error(res.statusText || 'Unsuccessful HTTP response');
    new_err.original = err;
    new_err.response = res;
    new_err.status = res.status;

    self.callback(err || new_err, res);
  });
}

/**
 * Mixin `Emitter`.
 */

Emitter(Request.prototype);

/**
 * Allow for extension
 */

Request.prototype.use = function(fn) {
  fn(this);
  return this;
}

/**
 * Set timeout to `ms`.
 *
 * @param {Number} ms
 * @return {Request} for chaining
 * @api public
 */

Request.prototype.timeout = function(ms){
  this._timeout = ms;
  return this;
};

/**
 * Clear previous timeout.
 *
 * @return {Request} for chaining
 * @api public
 */

Request.prototype.clearTimeout = function(){
  this._timeout = 0;
  clearTimeout(this._timer);
  return this;
};

/**
 * Abort the request, and clear potential timeout.
 *
 * @return {Request}
 * @api public
 */

Request.prototype.abort = function(){
  if (this.aborted) return;
  this.aborted = true;
  this.xhr.abort();
  this.clearTimeout();
  this.emit('abort');
  return this;
};

/**
 * Set header `field` to `val`, or multiple fields with one object.
 *
 * Examples:
 *
 *      req.get('/')
 *        .set('Accept', 'application/json')
 *        .set('X-API-Key', 'foobar')
 *        .end(callback);
 *
 *      req.get('/')
 *        .set({ Accept: 'application/json', 'X-API-Key': 'foobar' })
 *        .end(callback);
 *
 * @param {String|Object} field
 * @param {String} val
 * @return {Request} for chaining
 * @api public
 */

Request.prototype.set = function(field, val){
  if (isObject(field)) {
    for (var key in field) {
      this.set(key, field[key]);
    }
    return this;
  }
  this._header[field.toLowerCase()] = val;
  this.header[field] = val;
  return this;
};

/**
 * Remove header `field`.
 *
 * Example:
 *
 *      req.get('/')
 *        .unset('User-Agent')
 *        .end(callback);
 *
 * @param {String} field
 * @return {Request} for chaining
 * @api public
 */

Request.prototype.unset = function(field){
  delete this._header[field.toLowerCase()];
  delete this.header[field];
  return this;
};

/**
 * Get case-insensitive header `field` value.
 *
 * @param {String} field
 * @return {String}
 * @api private
 */

Request.prototype.getHeader = function(field){
  return this._header[field.toLowerCase()];
};

/**
 * Set Content-Type to `type`, mapping values from `request.types`.
 *
 * Examples:
 *
 *      superagent.types.xml = 'application/xml';
 *
 *      request.post('/')
 *        .type('xml')
 *        .send(xmlstring)
 *        .end(callback);
 *
 *      request.post('/')
 *        .type('application/xml')
 *        .send(xmlstring)
 *        .end(callback);
 *
 * @param {String} type
 * @return {Request} for chaining
 * @api public
 */

Request.prototype.type = function(type){
  this.set('Content-Type', request.types[type] || type);
  return this;
};

/**
 * Set Accept to `type`, mapping values from `request.types`.
 *
 * Examples:
 *
 *      superagent.types.json = 'application/json';
 *
 *      request.get('/agent')
 *        .accept('json')
 *        .end(callback);
 *
 *      request.get('/agent')
 *        .accept('application/json')
 *        .end(callback);
 *
 * @param {String} accept
 * @return {Request} for chaining
 * @api public
 */

Request.prototype.accept = function(type){
  this.set('Accept', request.types[type] || type);
  return this;
};

/**
 * Set Authorization field value with `user` and `pass`.
 *
 * @param {String} user
 * @param {String} pass
 * @return {Request} for chaining
 * @api public
 */

Request.prototype.auth = function(user, pass){
  var str = btoa(user + ':' + pass);
  this.set('Authorization', 'Basic ' + str);
  return this;
};

/**
* Add query-string `val`.
*
* Examples:
*
*   request.get('/shoes')
*     .query('size=10')
*     .query({ color: 'blue' })
*
* @param {Object|String} val
* @return {Request} for chaining
* @api public
*/

Request.prototype.query = function(val){
  if ('string' != typeof val) val = serialize(val);
  if (val) this._query.push(val);
  return this;
};

/**
 * Write the field `name` and `val` for "multipart/form-data"
 * request bodies.
 *
 * ``` js
 * request.post('/upload')
 *   .field('foo', 'bar')
 *   .end(callback);
 * ```
 *
 * @param {String} name
 * @param {String|Blob|File} val
 * @return {Request} for chaining
 * @api public
 */

Request.prototype.field = function(name, val){
  if (!this._formData) this._formData = new root.FormData();
  this._formData.append(name, val);
  return this;
};

/**
 * Queue the given `file` as an attachment to the specified `field`,
 * with optional `filename`.
 *
 * ``` js
 * request.post('/upload')
 *   .attach(new Blob(['<a id="a"><b id="b">hey!</b></a>'], { type: "text/html"}))
 *   .end(callback);
 * ```
 *
 * @param {String} field
 * @param {Blob|File} file
 * @param {String} filename
 * @return {Request} for chaining
 * @api public
 */

Request.prototype.attach = function(field, file, filename){
  if (!this._formData) this._formData = new root.FormData();
  this._formData.append(field, file, filename);
  return this;
};

/**
 * Send `data`, defaulting the `.type()` to "json" when
 * an object is given.
 *
 * Examples:
 *
 *       // querystring
 *       request.get('/search')
 *         .end(callback)
 *
 *       // multiple data "writes"
 *       request.get('/search')
 *         .send({ search: 'query' })
 *         .send({ range: '1..5' })
 *         .send({ order: 'desc' })
 *         .end(callback)
 *
 *       // manual json
 *       request.post('/user')
 *         .type('json')
 *         .send('{"name":"tj"})
 *         .end(callback)
 *
 *       // auto json
 *       request.post('/user')
 *         .send({ name: 'tj' })
 *         .end(callback)
 *
 *       // manual x-www-form-urlencoded
 *       request.post('/user')
 *         .type('form')
 *         .send('name=tj')
 *         .end(callback)
 *
 *       // auto x-www-form-urlencoded
 *       request.post('/user')
 *         .type('form')
 *         .send({ name: 'tj' })
 *         .end(callback)
 *
 *       // defaults to x-www-form-urlencoded
  *      request.post('/user')
  *        .send('name=tobi')
  *        .send('species=ferret')
  *        .end(callback)
 *
 * @param {String|Object} data
 * @return {Request} for chaining
 * @api public
 */

Request.prototype.send = function(data){
  var obj = isObject(data);
  var type = this.getHeader('Content-Type');

  // merge
  if (obj && isObject(this._data)) {
    for (var key in data) {
      this._data[key] = data[key];
    }
  } else if ('string' == typeof data) {
    if (!type) this.type('form');
    type = this.getHeader('Content-Type');
    if ('application/x-www-form-urlencoded' == type) {
      this._data = this._data
        ? this._data + '&' + data
        : data;
    } else {
      this._data = (this._data || '') + data;
    }
  } else {
    this._data = data;
  }

  if (!obj || isHost(data)) return this;
  if (!type) this.type('json');
  return this;
};

/**
 * Invoke the callback with `err` and `res`
 * and handle arity check.
 *
 * @param {Error} err
 * @param {Response} res
 * @api private
 */

Request.prototype.callback = function(err, res){
  var fn = this._callback;
  this.clearTimeout();
  fn(err, res);
};

/**
 * Invoke callback with x-domain error.
 *
 * @api private
 */

Request.prototype.crossDomainError = function(){
  var err = new Error('Origin is not allowed by Access-Control-Allow-Origin');
  err.crossDomain = true;
  this.callback(err);
};

/**
 * Invoke callback with timeout error.
 *
 * @api private
 */

Request.prototype.timeoutError = function(){
  var timeout = this._timeout;
  var err = new Error('timeout of ' + timeout + 'ms exceeded');
  err.timeout = timeout;
  this.callback(err);
};

/**
 * Enable transmission of cookies with x-domain requests.
 *
 * Note that for this to work the origin must not be
 * using "Access-Control-Allow-Origin" with a wildcard,
 * and also must set "Access-Control-Allow-Credentials"
 * to "true".
 *
 * @api public
 */

Request.prototype.withCredentials = function(){
  this._withCredentials = true;
  return this;
};

/**
 * Initiate request, invoking callback `fn(res)`
 * with an instanceof `Response`.
 *
 * @param {Function} fn
 * @return {Request} for chaining
 * @api public
 */

Request.prototype.end = function(fn){
  var self = this;
  var xhr = this.xhr = request.getXHR();
  var query = this._query.join('&');
  var timeout = this._timeout;
  var data = this._formData || this._data;

  // store callback
  this._callback = fn || noop;

  // state change
  xhr.onreadystatechange = function(){
    if (4 != xhr.readyState) return;

    // In IE9, reads to any property (e.g. status) off of an aborted XHR will
    // result in the error "Could not complete the operation due to error c00c023f"
    var status;
    try { status = xhr.status } catch(e) { status = 0; }

    if (0 == status) {
      if (self.timedout) return self.timeoutError();
      if (self.aborted) return;
      return self.crossDomainError();
    }
    self.emit('end');
  };

  // progress
  try {
    if (xhr.upload && this.hasListeners('progress')) {
      xhr.upload.onprogress = function(e){
        e.percent = e.loaded / e.total * 100;
        self.emit('progress', e);
      };
    }
  } catch(e) {
    // Accessing xhr.upload fails in IE from a web worker, so just pretend it doesn't exist.
    // Reported here:
    // https://connect.microsoft.com/IE/feedback/details/837245/xmlhttprequest-upload-throws-invalid-argument-when-used-from-web-worker-context
  }

  // timeout
  if (timeout && !this._timer) {
    this._timer = setTimeout(function(){
      self.timedout = true;
      self.abort();
    }, timeout);
  }

  // querystring
  if (query) {
    query = request.serializeObject(query);
    this.url += ~this.url.indexOf('?')
      ? '&' + query
      : '?' + query;
  }

  // initiate request
  xhr.open(this.method, this.url, true);

  // CORS
  if (this._withCredentials) xhr.withCredentials = true;

  // body
  if ('GET' != this.method && 'HEAD' != this.method && 'string' != typeof data && !isHost(data)) {
    // serialize stuff
    var serialize = request.serialize[this.getHeader('Content-Type')];
    if (serialize) data = serialize(data);
  }

  // set header fields
  for (var field in this.header) {
    if (null == this.header[field]) continue;
    xhr.setRequestHeader(field, this.header[field]);
  }

  // send stuff
  this.emit('request', this);
  xhr.send(data);
  return this;
};

/**
 * Expose `Request`.
 */

request.Request = Request;

/**
 * Issue a request:
 *
 * Examples:
 *
 *    request('GET', '/users').end(callback)
 *    request('/users').end(callback)
 *    request('/users', callback)
 *
 * @param {String} method
 * @param {String|Function} url or callback
 * @return {Request}
 * @api public
 */

function request(method, url) {
  // callback
  if ('function' == typeof url) {
    return new Request('GET', method).end(url);
  }

  // url first
  if (1 == arguments.length) {
    return new Request('GET', method);
  }

  return new Request(method, url);
}

/**
 * GET `url` with optional callback `fn(res)`.
 *
 * @param {String} url
 * @param {Mixed|Function} data or fn
 * @param {Function} fn
 * @return {Request}
 * @api public
 */

request.get = function(url, data, fn){
  var req = request('GET', url);
  if ('function' == typeof data) fn = data, data = null;
  if (data) req.query(data);
  if (fn) req.end(fn);
  return req;
};

/**
 * HEAD `url` with optional callback `fn(res)`.
 *
 * @param {String} url
 * @param {Mixed|Function} data or fn
 * @param {Function} fn
 * @return {Request}
 * @api public
 */

request.head = function(url, data, fn){
  var req = request('HEAD', url);
  if ('function' == typeof data) fn = data, data = null;
  if (data) req.send(data);
  if (fn) req.end(fn);
  return req;
};

/**
 * DELETE `url` with optional callback `fn(res)`.
 *
 * @param {String} url
 * @param {Function} fn
 * @return {Request}
 * @api public
 */

request.del = function(url, fn){
  var req = request('DELETE', url);
  if (fn) req.end(fn);
  return req;
};

/**
 * PATCH `url` with optional `data` and callback `fn(res)`.
 *
 * @param {String} url
 * @param {Mixed} data
 * @param {Function} fn
 * @return {Request}
 * @api public
 */

request.patch = function(url, data, fn){
  var req = request('PATCH', url);
  if ('function' == typeof data) fn = data, data = null;
  if (data) req.send(data);
  if (fn) req.end(fn);
  return req;
};

/**
 * POST `url` with optional `data` and callback `fn(res)`.
 *
 * @param {String} url
 * @param {Mixed} data
 * @param {Function} fn
 * @return {Request}
 * @api public
 */

request.post = function(url, data, fn){
  var req = request('POST', url);
  if ('function' == typeof data) fn = data, data = null;
  if (data) req.send(data);
  if (fn) req.end(fn);
  return req;
};

/**
 * PUT `url` with optional `data` and callback `fn(res)`.
 *
 * @param {String} url
 * @param {Mixed|Function} data or fn
 * @param {Function} fn
 * @return {Request}
 * @api public
 */

request.put = function(url, data, fn){
  var req = request('PUT', url);
  if ('function' == typeof data) fn = data, data = null;
  if (data) req.send(data);
  if (fn) req.end(fn);
  return req;
};

/**
 * Expose `request`.
 */

module.exports = request;

}, {"emitter":23,"reduce":24}],
23: [function(require, module, exports) {

/**
 * Expose `Emitter`.
 */

if (typeof module !== 'undefined') {
  module.exports = Emitter;
}

/**
 * Initialize a new `Emitter`.
 *
 * @api public
 */

function Emitter(obj) {
  if (obj) return mixin(obj);
};

/**
 * Mixin the emitter properties.
 *
 * @param {Object} obj
 * @return {Object}
 * @api private
 */

function mixin(obj) {
  for (var key in Emitter.prototype) {
    obj[key] = Emitter.prototype[key];
  }
  return obj;
}

/**
 * Listen on the given `event` with `fn`.
 *
 * @param {String} event
 * @param {Function} fn
 * @return {Emitter}
 * @api public
 */

Emitter.prototype.on =
Emitter.prototype.addEventListener = function(event, fn){
  this._callbacks = this._callbacks || {};
  (this._callbacks['$' + event] = this._callbacks['$' + event] || [])
    .push(fn);
  return this;
};

/**
 * Adds an `event` listener that will be invoked a single
 * time then automatically removed.
 *
 * @param {String} event
 * @param {Function} fn
 * @return {Emitter}
 * @api public
 */

Emitter.prototype.once = function(event, fn){
  function on() {
    this.off(event, on);
    fn.apply(this, arguments);
  }

  on.fn = fn;
  this.on(event, on);
  return this;
};

/**
 * Remove the given callback for `event` or all
 * registered callbacks.
 *
 * @param {String} event
 * @param {Function} fn
 * @return {Emitter}
 * @api public
 */

Emitter.prototype.off =
Emitter.prototype.removeListener =
Emitter.prototype.removeAllListeners =
Emitter.prototype.removeEventListener = function(event, fn){
  this._callbacks = this._callbacks || {};

  // all
  if (0 == arguments.length) {
    this._callbacks = {};
    return this;
  }

  // specific event
  var callbacks = this._callbacks['$' + event];
  if (!callbacks) return this;

  // remove all handlers
  if (1 == arguments.length) {
    delete this._callbacks['$' + event];
    return this;
  }

  // remove specific handler
  var cb;
  for (var i = 0; i < callbacks.length; i++) {
    cb = callbacks[i];
    if (cb === fn || cb.fn === fn) {
      callbacks.splice(i, 1);
      break;
    }
  }
  return this;
};

/**
 * Emit `event` with the given args.
 *
 * @param {String} event
 * @param {Mixed} ...
 * @return {Emitter}
 */

Emitter.prototype.emit = function(event){
  this._callbacks = this._callbacks || {};
  var args = [].slice.call(arguments, 1)
    , callbacks = this._callbacks['$' + event];

  if (callbacks) {
    callbacks = callbacks.slice(0);
    for (var i = 0, len = callbacks.length; i < len; ++i) {
      callbacks[i].apply(this, args);
    }
  }

  return this;
};

/**
 * Return array of callbacks for `event`.
 *
 * @param {String} event
 * @return {Array}
 * @api public
 */

Emitter.prototype.listeners = function(event){
  this._callbacks = this._callbacks || {};
  return this._callbacks['$' + event] || [];
};

/**
 * Check if this emitter has `event` handlers.
 *
 * @param {String} event
 * @return {Boolean}
 * @api public
 */

Emitter.prototype.hasListeners = function(event){
  return !! this.listeners(event).length;
};

}, {}],
24: [function(require, module, exports) {

/**
 * Reduce `arr` with `fn`.
 *
 * @param {Array} arr
 * @param {Function} fn
 * @param {Mixed} initial
 *
 * TODO: combatible error handling?
 */

module.exports = function(arr, fn, initial){  
  var idx = 0;
  var len = arr.length;
  var curr = arguments.length == 3
    ? initial
    : arr[idx++];

  while (idx < len) {
    curr = fn.call(null, curr, arr[idx], ++idx, arr);
  }
  
  return curr;
};
}, {}],
3: [function(require, module, exports) {
(function(rootScope) {
  
  // original Array methods
  var
    arrayPop = Array.prototype.pop,
    arrayPush = Array.prototype.push,
    arrayReverse = Array.prototype.reverse,
    arrayShift = Array.prototype.shift,
    arraySort = Array.prototype.sort,
    arraySplice = Array.prototype.splice,
    arrayUnshift = Array.prototype.unshift;
  
  /**
   * Constructor.
   */
  function Arr() {
    var instance = this;
    
    if (! instance instanceof Arr) {
      instance = new Arr();
    }
        
    if (arguments.length === 1 && typeof arguments[0] === 'number' && arguments[0] % 1 === 0) {
      arrayPush.apply(instance, new Array(arguments[0]));
    } else {
      arrayPush.apply(instance, arguments);
    }
    
    instance.events = {};
    
    return instance;
  };
  
  // Attach prototype
  Arr.prototype = [];
  
  /**
   * Attached events.
   */
  Arr.prototype.events = {};
   
  /**
   * Get value by index.
   */
  Arr.prototype.get = function(index, defaultValue) {
    return typeof this[index] === 'undefined' ? defaultValue: this[index];
  };
   
  /**
   * Attach event handler.
   */
  Arr.prototype.on = function(eventName, handler) {
    if (typeof this.events[eventName] === 'undefined') {
      this.events[eventName] = [];
    }

    this.events[eventName].push(handler);

    return this;
  };
   
  /**
   * Trigger event.
   */
  Arr.prototype.trigger = function(eventName, args) {
    args = args || [];

    if (eventName instanceof Array) {
      for (var k=0, klen=eventName.length; k<klen; k++) {
        if (typeof this.events[eventName[k]] === 'undefined') {
          continue;
        }
       
        for (var i=0,len=this.events[eventName[k]].length; i<len; i++) {
          this.events[eventName[k]][i].apply(this, [args]);
        }
      }
    } else {
      for (var i=0,len=this.events[eventName].length; i<len; i++) {
        this.events[eventName][i].apply(this, [args]);
      }
    }

    return this;
  };
   
  /**
   * Update items by handler.
   */
  Arr.prototype.update = function(handler) {
    if (! handler instanceof Function) {
      throw new Error('handler should be an Function');
    }
    
    var oldValue, newValue, i, result = [];
   
    for (i=0,len=this.length; i<len; i++) {
      oldValue = this[i];
      newValue = handler.apply(this, [oldValue, i]);
      
      if (typeof newValue !== 'undefined') {
        this[i] = newValue;
        result.push(newValue);
      }
    }
   
    if (result.length > 0) {
      if (typeof this.events.change !== 'undefined' || typeof this.events.update !== 'undefined') {
        this.trigger(['change', 'update'], {
          type: 'update',
          items: result
        });
      }
    }
   
    return this;
  };
  
  /**
   * Insert array of items.
   */
  Arr.prototype.insert = function(items) {
    if (! items instanceof Array) {
      throw new Error('items should be an Array');
    }
    
    arrayPush.apply(this, items);

    if (typeof this.events.change !== 'undefined' || typeof this.events.insert !== 'undefined') {
      this.trigger(['change', 'insert'], {
        type: 'insert',
        items: items
      });
    }

    return this;
  };
  
  /**
   * Remove items by handler.
   */
  Arr.prototype.remove = function(handler) {
    if (typeof handler === 'undefined') { // drop all items
      if (this.length > 0) {
        this.splice(0, this.length);
      }
      
      return this;
    }
    
    if (! handler instanceof Function) {
      throw new Error('handler should be an Function');
    }
    
    var result = [], stay = [], i;

    for (i=0, len=this.length; i<len; i++) {
      isRemove = handler.apply(this, [this[i], i]);
      
      if (isRemove === true) {
        result.push(this[i]);
      } else {
        stay.push(this[i]);
      }
    }

    arraySplice.apply(this, [0, this.length]);
    arrayPush.apply(this, stay);

    if (result.length > 0) {
      if (typeof this.events.change !== 'undefined' || typeof this.events.remove !== 'undefined') {
        this.trigger(['change', 'remove'], {
          type: 'remove',
          items: result
        });
      }
    }
   
    return this;
  };
  
  /**
   * Set value by index.
   */
  Arr.prototype.set = function(index, value) {
    if (! index instanceof Number) {
      throw new Error('index should be an Number');
    }
    
    this[index] = value;

    if (typeof this.events.change !== 'undefined' || typeof this.events.update !== 'undefined') {
      this.trigger(['change', 'update'], {
        type: 'update',
        items: [this[index]]
      });
    }

    return this;
  };
   
  /**
   * Removes the last element from an array and returns that element.
   */
  Arr.prototype.pop = function() {
    var result = arrayPop.apply(this, arguments);

    if (typeof this.events.change !== 'undefined' || typeof this.events.remove !== 'undefined' ) {
      this.trigger(['change', 'remove'], {
        type: 'remove',
        items: [result]
      });
    }

    return result;
  };
   
  /**
   * Adds one or more elements to the end of an array and returns the new length of the array.
   */
  Arr.prototype.push = function() {
    var result = arrayPush.apply(this, arguments);

    if (typeof this.events.change !== 'undefined' || typeof this.events.insert !== 'undefined' ) {
      this.trigger(['change', 'insert'], {
        type: 'insert',
        items: Array.prototype.slice.call(arguments, 0)
      });
    }

    return result;
  };
   
  /**
   * Reverses the order of the elements of an array — the first becomes the last, and the last becomes the first.
   */
  Arr.prototype.reverse = function() {
    var result = arrayReverse.apply(this, arguments);

    if (typeof this.events.change !== 'undefined' || typeof this.events.update !== 'undefined' ) {
      this.trigger(['change', 'update'], {
        type: 'update',
        items: Array.prototype.slice.call(result, 0)
      });
    }

    return result;
  };
   
  /**
   * Removes the first element from an array and returns that element.
   */
  Arr.prototype.shift = function() {
    var result = arrayShift.apply(this, arguments);

    if (typeof this.events.change !== 'undefined' || typeof this.events.remove !== 'undefined' ) {
      this.trigger(['change', 'remove'], {
        type: 'remove',
        items: [result]
      });
    }

    return result;
  };
   
  /**
   * Sorts the elements of an array in place and returns the array.
   */
  Arr.prototype.sort = function() {
    var result = arraySort.apply(this, arguments);

    if (typeof this.events.change !== 'undefined' || typeof this.events.update !== 'undefined' ) {
      this.trigger(['change', 'update'], {
        type: 'update',
        items: Array.prototype.slice.call(result, 0)
      });
    }

    return result;
  };
   
  /**
   * Adds and/or removes elements from an array.
   */
  Arr.prototype.splice = function() {
    var result = arraySplice.apply(this, arguments);

    if (result.length > 0) {
      if (typeof this.events.change !== 'undefined' || typeof this.events.remove !== 'undefined' ) {
        this.trigger(['change', 'remove'], {
          type: 'remove',
          items: Array.prototype.slice.call(result, 0)
        });
      }
    }

    if (arguments.length > 2) {
      if (typeof this.events.change !== 'undefined' || typeof this.events.insert !== 'undefined' ) {
        this.trigger(['change', 'insert'], {
          type: 'insert',
          items: Array.prototype.slice.call(arguments, 2)
        });
      }
    }

    return result;
  };
   
  /**
   * Adds one or more elements to the front of an array and returns the new length of the array.
   */
  Arr.prototype.unshift = function() {
    var result = arrayUnshift.apply(this, arguments);

    if (typeof this.events.change !== 'undefined' || typeof this.events.insert !== 'undefined' ) {
      this.trigger(['change', 'insert'], {
        type: 'insert',
        items: [result]
      });
    }

    return result;
  };
  
  // exports

  if (typeof module !== 'undefined') {
    module.exports = Arr;
  } else {
    rootScope.Arr = Arr;
  }
})(this);

}, {}],
4: [function(require, module, exports) {
var Template = require('hogan-runtime').Template;module.exports = new Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<div class=\"filepicker\">");t.b("\n" + i);t.b("  <div id=\"filepicker-dialog\" class=\"filepicker-dialog\">");t.b("\n" + i);t.b("    <div class=\"filepicker-dialog-box\">");t.b("\n" + i);t.b("      <div class=\"filepicker-navbar\">");t.b("\n" + i);t.b("        <div class=\"filepicker-title\">Select a photo source</div>");t.b("\n" + i);t.b("        <a id=\"filepicker-cancel-btn\" class=\"filepicker-navbar-right-btn\">Cancel</a>");t.b("\n" + i);t.b("      </div>");t.b("\n");t.b("\n" + i);t.b("      <div id=\"filepicker-sources-list\" class=\"filepicker-list\"></div>");t.b("\n");t.b("\n" + i);t.b("      <div id=\"filepicker-dialog-loading\" class=\"filepicker-loading\">Loading&hellip;</div>");t.b("\n" + i);t.b("    </div>");t.b("\n" + i);t.b("  </div>");t.b("\n" + i);t.b("</div>");return t.fl(); },partials: {}, subs: {  }});
}, {"hogan-runtime":25}],
25: [function(require, module, exports) {
/*
 *  Copyright 2011 Twitter, Inc.
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *  http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

var Hogan = {};

(function (Hogan) {
  Hogan.Template = function (codeObj, text, compiler, options) {
    codeObj = codeObj || {};
    this.r = codeObj.code || this.r;
    this.c = compiler;
    this.options = options || {};
    this.text = text || '';
    this.partials = codeObj.partials || {};
    this.subs = codeObj.subs || {};
    this.buf = '';
  }

  Hogan.Template.prototype = {
    // render: replaced by generated code.
    r: function (context, partials, indent) { return ''; },

    // variable escaping
    v: hoganEscape,

    // triple stache
    t: coerceToString,

    render: function render(context, partials, indent) {
      return this.ri([context], partials || {}, indent);
    },

    // render internal -- a hook for overrides that catches partials too
    ri: function (context, partials, indent) {
      return this.r(context, partials, indent);
    },

    // ensurePartial
    ep: function(symbol, partials) {
      var partial = this.partials[symbol];

      // check to see that if we've instantiated this partial before
      var template = partials[partial.name];
      if (partial.instance && partial.base == template) {
        return partial.instance;
      }

      if (typeof template == 'string') {
        if (!this.c) {
          throw new Error("No compiler available.");
        }
        template = this.c.compile(template, this.options);
      }

      if (!template) {
        return null;
      }

      // We use this to check whether the partials dictionary has changed
      this.partials[symbol].base = template;

      if (partial.subs) {
        // Make sure we consider parent template now
        if (!partials.stackText) partials.stackText = {};
        for (key in partial.subs) {
          if (!partials.stackText[key]) {
            partials.stackText[key] = (this.activeSub !== undefined && partials.stackText[this.activeSub]) ? partials.stackText[this.activeSub] : this.text;
          }
        }
        template = createSpecializedPartial(template, partial.subs, partial.partials,
          this.stackSubs, this.stackPartials, partials.stackText);
      }
      this.partials[symbol].instance = template;

      return template;
    },

    // tries to find a partial in the current scope and render it
    rp: function(symbol, context, partials, indent) {
      var partial = this.ep(symbol, partials);
      if (!partial) {
        return '';
      }

      return partial.ri(context, partials, indent);
    },

    // render a section
    rs: function(context, partials, section) {
      var tail = context[context.length - 1];

      if (!isArray(tail)) {
        section(context, partials, this);
        return;
      }

      for (var i = 0; i < tail.length; i++) {
        context.push(tail[i]);
        section(context, partials, this);
        context.pop();
      }
    },

    // maybe start a section
    s: function(val, ctx, partials, inverted, start, end, tags) {
      var pass;

      if (isArray(val) && val.length === 0) {
        return false;
      }

      if (typeof val == 'function') {
        val = this.ms(val, ctx, partials, inverted, start, end, tags);
      }

      pass = !!val;

      if (!inverted && pass && ctx) {
        ctx.push((typeof val == 'object') ? val : ctx[ctx.length - 1]);
      }

      return pass;
    },

    // find values with dotted names
    d: function(key, ctx, partials, returnFound) {
      var found,
          names = key.split('.'),
          val = this.f(names[0], ctx, partials, returnFound),
          doModelGet = this.options.modelGet,
          cx = null;

      if (key === '.' && isArray(ctx[ctx.length - 2])) {
        val = ctx[ctx.length - 1];
      } else {
        for (var i = 1; i < names.length; i++) {
          found = findInScope(names[i], val, doModelGet);
          if (found !== undefined) {
            cx = val;
            val = found;
          } else {
            val = '';
          }
        }
      }

      if (returnFound && !val) {
        return false;
      }

      if (!returnFound && typeof val == 'function') {
        ctx.push(cx);
        val = this.mv(val, ctx, partials);
        ctx.pop();
      }

      return val;
    },

    // find values with normal names
    f: function(key, ctx, partials, returnFound) {
      var val = false,
          v = null,
          found = false,
          doModelGet = this.options.modelGet;

      for (var i = ctx.length - 1; i >= 0; i--) {
        v = ctx[i];
        val = findInScope(key, v, doModelGet);
        if (val !== undefined) {
          found = true;
          break;
        }
      }

      if (!found) {
        return (returnFound) ? false : "";
      }

      if (!returnFound && typeof val == 'function') {
        val = this.mv(val, ctx, partials);
      }

      return val;
    },

    // higher order templates
    ls: function(func, cx, partials, text, tags) {
      var oldTags = this.options.delimiters;

      this.options.delimiters = tags;
      this.b(this.ct(coerceToString(func.call(cx, text)), cx, partials));
      this.options.delimiters = oldTags;

      return false;
    },

    // compile text
    ct: function(text, cx, partials) {
      if (this.options.disableLambda) {
        throw new Error('Lambda features disabled.');
      }
      return this.c.compile(text, this.options).render(cx, partials);
    },

    // template result buffering
    b: function(s) { this.buf += s; },

    fl: function() { var r = this.buf; this.buf = ''; return r; },

    // method replace section
    ms: function(func, ctx, partials, inverted, start, end, tags) {
      var textSource,
          cx = ctx[ctx.length - 1],
          result = func.call(cx);

      if (typeof result == 'function') {
        if (inverted) {
          return true;
        } else {
          textSource = (this.activeSub && this.subsText && this.subsText[this.activeSub]) ? this.subsText[this.activeSub] : this.text;
          return this.ls(result, cx, partials, textSource.substring(start, end), tags);
        }
      }

      return result;
    },

    // method replace variable
    mv: function(func, ctx, partials) {
      var cx = ctx[ctx.length - 1];
      var result = func.call(cx);

      if (typeof result == 'function') {
        return this.ct(coerceToString(result.call(cx)), cx, partials);
      }

      return result;
    },

    sub: function(name, context, partials, indent) {
      var f = this.subs[name];
      if (f) {
        this.activeSub = name;
        f(context, partials, this, indent);
        this.activeSub = false;
      }
    }

  };

  //Find a key in an object
  function findInScope(key, scope, doModelGet) {
    var val;

    if (scope && typeof scope == 'object') {

      if (scope[key] !== undefined) {
        val = scope[key];

      // try lookup with get for backbone or similar model data
      } else if (doModelGet && scope.get && typeof scope.get == 'function') {
        val = scope.get(key);
      }
    }

    return val;
  }

  function createSpecializedPartial(instance, subs, partials, stackSubs, stackPartials, stackText) {
    function PartialTemplate() {};
    PartialTemplate.prototype = instance;
    function Substitutions() {};
    Substitutions.prototype = instance.subs;
    var key;
    var partial = new PartialTemplate();
    partial.subs = new Substitutions();
    partial.subsText = {};  //hehe. substext.
    partial.buf = '';

    stackSubs = stackSubs || {};
    partial.stackSubs = stackSubs;
    partial.subsText = stackText;
    for (key in subs) {
      if (!stackSubs[key]) stackSubs[key] = subs[key];
    }
    for (key in stackSubs) {
      partial.subs[key] = stackSubs[key];
    }

    stackPartials = stackPartials || {};
    partial.stackPartials = stackPartials;
    for (key in partials) {
      if (!stackPartials[key]) stackPartials[key] = partials[key];
    }
    for (key in stackPartials) {
      partial.partials[key] = stackPartials[key];
    }

    return partial;
  }

  var rAmp = /&/g,
      rLt = /</g,
      rGt = />/g,
      rApos = /\'/g,
      rQuot = /\"/g,
      hChars = /[&<>\"\']/;

  function coerceToString(val) {
    return String((val === null || val === undefined) ? '' : val);
  }

  function hoganEscape(str) {
    str = coerceToString(str);
    return hChars.test(str) ?
      str
        .replace(rAmp, '&amp;')
        .replace(rLt, '&lt;')
        .replace(rGt, '&gt;')
        .replace(rApos, '&#39;')
        .replace(rQuot, '&quot;') :
      str;
  }

  var isArray = Array.isArray || function(a) {
    return Object.prototype.toString.call(a) === '[object Array]';
  };

})(typeof exports !== 'undefined' ? exports : Hogan);

}, {}],
5: [function(require, module, exports) {
var Template = require('hogan-runtime').Template;module.exports = new Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<div class=\"filepicker-list-item\" data-id=\"");t.b(t.v(t.f("id",c,p,0)));t.b("\">");t.b("\n" + i);t.b("  <span class=\"icon1 icon-");t.b(t.v(t.f("id",c,p,0)));t.b("\"></span>");t.b("\n" + i);t.b("  <span class=\"name\">");t.b(t.v(t.f("name",c,p,0)));t.b("</span>");t.b("\n" + i);t.b("</div>");return t.fl(); },partials: {}, subs: {  }});
}, {"hogan-runtime":25}],
6: [function(require, module, exports) {
var Template = require('hogan-runtime').Template;module.exports = new Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<div class=\"photopicker\" id=\"saveLayer\">");t.b("\n" + i);t.b("    <div id=\"save-dialog\" class=\"photopicker-dialog\">");t.b("\n" + i);t.b("        <div class=\"save-dialog-box\">");t.b("\n" + i);t.b("            <div class=\"photopicker-navbar\">");t.b("\n" + i);t.b("                <div class=\"photopicker-title\">Sharing Options</div>");t.b("\n" + i);t.b("                <a id=\"save-cancel-btn\" class=\"photopicker-navbar-right-btn\">Close</a>");t.b("\n" + i);t.b("            </div>");t.b("\n" + i);t.b("            <div id=\"saveOptionContainer\">");t.b("\n" + i);t.b("                <div id=\"savePhotoContainer\">");t.b("\n" + i);t.b("                    <img crossOrigin=\"Anonymous\" id=\"saveImg\"/>");t.b("\n" + i);t.b("                    <progress class=\"buttonProgress\" id=\"saveMainProgress\" value=\"0\" max=\".5\"></progress>");t.b("\n" + i);t.b("                </div>");t.b("\n" + i);t.b("                <div id=\"saveTopColumn\">");t.b("\n" + i);t.b("                    <div id=\"saveCaptionContainer\">");t.b("\n" + i);t.b("                        <span class=\"bold\">Caption</span>");t.b("\n" + i);t.b("                        <textarea rows=\"2\" id=\"saveCaption\"></textarea>");t.b("\n" + i);t.b("                    </div>");t.b("\n" + i);t.b("                    <div id=\"imageQualityContainer\">");t.b("\n" + i);t.b("                        <span class=\"bold\">Image Quality:</span>");t.b("\n" + i);t.b("                        <span class=\"image_quality\" id=\"standardText\">Standard</span><input type=\"radio\" content=\"Standard\"");t.b("\n" + i);t.b("                                                                          value=\"standard\" id=\"standard\" name=\"quality\" checked=\"true\">");t.b("\n" + i);t.b("                        <span class=\"image_quality\"><img");t.b("\n" + i);t.b("                                src=\"https://static.pizap.com/pizapfiles/images/pizap-pro-icon-74x74.png\" width=\"22\"");t.b("\n" + i);t.b("                                height=\"22\"/> High Quality</span><input type=\"radio\" content=\"HiDef\" value=\"hidef\" id=\"hidef\" name=\"quality\">");t.b("\n" + i);t.b("                        <span id=\"progressNumber\"></span>");t.b("\n" + i);t.b("                    </div>");t.b("\n" + i);t.b("                </div>");t.b("\n" + i);t.b("            </div>");t.b("\n" + i);t.b("            <div id=\"saveDestinations\">");t.b("\n" + i);t.b("                <div class=\"saveDestinationColumn\">");t.b("\n" + i);t.b("                    <div class=\"saveDestination\">");t.b("\n" + i);t.b("                        <button id=\"pizapSave\">");t.b("\n" + i);t.b("                            <img src=\"/images/pizap-icon.jpg\" height=\"25\"");t.b("\n" + i);t.b("                                 width=\"25\">");t.b("\n" + i);t.b("                            <span class=\"destinationName\">piZap</span>");t.b("\n" + i);t.b("                            <img class=\"saveCheck\" src=\"/images/check.png\" id=\"savePizapCheck\"/>");t.b("\n" + i);t.b("                            <progress class=\"buttonProgress\" id=\"savePizapProgress\" value=\"0\"></progress>");t.b("\n" + i);t.b("                        </button>");t.b("\n" + i);t.b("                    </div>");t.b("\n");t.b("\n" + i);t.b("                    <div class=\"saveDestination\">");t.b("\n" + i);t.b("                        <button id=\"tumblrSave\">");t.b("\n" + i);t.b("                        <img src=\"//cdn.pizap.com/pizapfiles/images/tumblr-icon-20px.png\" height=\"25\" width=\"25\">");t.b("\n" + i);t.b("                        <span class=\"destinationName\">Tumblr</span>");t.b("\n" + i);t.b("                            <img class=\"saveCheck\" src=\"/images/check.png\" id=\"saveTumblrCheck\"/>");t.b("\n" + i);t.b("                            <progress class=\"buttonProgress\" id=\"saveTumblrProgress\" value=\"0\"></progress>");t.b("\n" + i);t.b("                        </button>");t.b("\n" + i);t.b("                    </div>");t.b("\n");t.b("\n" + i);t.b("                    <div class=\"saveDestination\">");t.b("\n" + i);t.b("                        <button id=\"heartItSave\">");t.b("\n" + i);t.b("                        <img src=\"/images/weheartit.jpg\" height=\"25\"");t.b("\n" + i);t.b("                             width=\"25\">");t.b("\n" + i);t.b("                        <span class=\"destinationName\">We Heart It</span>");t.b("\n" + i);t.b("                            <img class=\"saveCheck\" src=\"/images/check.png\" id=\"saveWeHeartItCheck\"/>");t.b("\n" + i);t.b("                            <progress class=\"buttonProgress\" id=\"saveWeHeartItProgress\" value=\"0\"></progress>");t.b("\n" + i);t.b("                        </button>");t.b("\n" + i);t.b("                    </div>");t.b("\n" + i);t.b("                </div>");t.b("\n" + i);t.b("                <div class=\"saveDestinationColumn\">");t.b("\n" + i);t.b("                    <div class=\"saveDestination\">");t.b("\n" + i);t.b("                        <button id=\"facebookSave\">");t.b("\n" + i);t.b("                        <img src=\"//cdn.pizap.com/pizapfiles/images/file_picker_icons/facebook.png\" height=\"25\"");t.b("\n" + i);t.b("                             width=\"25\">");t.b("\n" + i);t.b("                        <span class=\"destinationName\">Facebook</span>");t.b("\n" + i);t.b("                            <img class=\"saveCheck\" src=\"/images/check.png\" id=\"saveFacebookCheck\"/>");t.b("\n" + i);t.b("                            <progress class=\"buttonProgress\" id=\"saveFacebookProgress\" value=\"0\"></progress>");t.b("\n" + i);t.b("                        </button>");t.b("\n" + i);t.b("                    </div>");t.b("\n" + i);t.b("                    <div class=\"saveDestination\">");t.b("\n" + i);t.b("                        <button id=\"twitterSave\">");t.b("\n" + i);t.b("                        <img src=\"//cdn.pizap.com/pizapfiles/images/twitter-icon-20px.png\" height=\"25\" width=\"25\">");t.b("\n" + i);t.b("                        <span class=\"destinationName\">Twitter</span>");t.b("\n" + i);t.b("                            <img class=\"saveCheck\" src=\"/images/check.png\" id=\"saveTwitterCheck\"/>");t.b("\n" + i);t.b("                            <progress class=\"buttonProgress\" id=\"saveTwitterProgress\" value=\"0\"></progress>");t.b("\n" + i);t.b("                        </button>");t.b("\n" + i);t.b("                    </div>");t.b("\n" + i);t.b("                    <div class=\"saveDestination\">");t.b("\n" + i);t.b("                        <button id=\"pinterestSave\">");t.b("\n" + i);t.b("                        <img src=\"/images/pinterest.png\" height=\"25\"");t.b("\n" + i);t.b("                             width=\"25\">");t.b("\n" + i);t.b("                        <span class=\"destinationName\">Pinterest</span>");t.b("\n" + i);t.b("                            <img class=\"saveCheck\" src=\"/images/check.png\" id=\"savePinterestCheck\"/>");t.b("\n" + i);t.b("                            <progress class=\"buttonProgress\" id=\"savePinterestProgress\" value=\"0\"></progress>");t.b("\n" + i);t.b("                        </button>");t.b("\n" + i);t.b("                    </div>");t.b("\n" + i);t.b("                </div>");t.b("\n" + i);t.b("                <div class=\"saveDestinationColumn\">");t.b("\n" + i);t.b("                    <div class=\"saveDestination\">");t.b("\n" + i);t.b("                        <button id=\"dropboxSave\">");t.b("\n" + i);t.b("                            <img src=\"/images/dropbox.png\" height=\"25\"");t.b("\n" + i);t.b("                                 width=\"25\">");t.b("\n" + i);t.b("                            <span class=\"destinationName\">Dropbox</span>");t.b("\n" + i);t.b("                            <img class=\"saveCheck\" src=\"/images/check.png\" id=\"saveDropboxCheck\"/>");t.b("\n" + i);t.b("                            <progress class=\"buttonProgress\" id=\"saveDropboxProgress\" value=\"0\"></progress>");t.b("\n" + i);t.b("                        </button>");t.b("\n" + i);t.b("                    </div>");t.b("\n" + i);t.b("                    <div class=\"saveDestination\">");t.b("\n" + i);t.b("                        <button id=\"flickrSave\">");t.b("\n" + i);t.b("                            <img src=\"/images/flickr.png\" height=\"25\"");t.b("\n" + i);t.b("                                 width=\"25\">");t.b("\n" + i);t.b("                            <span class=\"destinationName\">Flickr</span>");t.b("\n" + i);t.b("                            <img class=\"saveCheck\" src=\"/images/check.png\" id=\"saveFlickrCheck\"/>");t.b("\n" + i);t.b("                             <progress class=\"buttonProgress\" id=\"saveFlickrProgress\" value=\"0\"></progress>");t.b("\n" + i);t.b("                        </button>");t.b("\n" + i);t.b("                    </div>");t.b("\n" + i);t.b("                    <div class=\"saveDestination\">");t.b("\n" + i);t.b("                        <button id=\"googleSave\">");t.b("\n" + i);t.b("                            <img src=\"/images/google-drive.png\" height=\"25\"");t.b("\n" + i);t.b("                                 width=\"25\">");t.b("\n" + i);t.b("                            <span class=\"destinationName\">Google Drive</span>");t.b("\n" + i);t.b("                            <img class=\"saveCheck\" src=\"/images/check.png\" id=\"saveGoogleCheck\"/>");t.b("\n" + i);t.b("                            <progress class=\"buttonProgress\" id=\"saveGoogleProgress\" value=\"0\"></progress>");t.b("\n" + i);t.b("                        </button>");t.b("\n" + i);t.b("                    </div>");t.b("\n" + i);t.b("                </div>");t.b("\n" + i);t.b("                <div class=\"saveDestinationColumn\">");t.b("\n" + i);t.b("                    <div class=\"saveDestination\">");t.b("\n" + i);t.b("                        <button id=\"emailSave\">");t.b("\n" + i);t.b("                            <img src=\"//cdn.pizap.com/pizapfiles/images/ic_email_grey600_24dp.png\" height=\"25\"");t.b("\n" + i);t.b("                                 width=\"25\">");t.b("\n" + i);t.b("                            <span class=\"destinationName\">Email</span>");t.b("\n" + i);t.b("                            <img class=\"saveCheck\" src=\"/images/check.png\" id=\"saveEmailCheck\"/>");t.b("\n" + i);t.b("                            <progress class=\"buttonProgress\" id=\"saveEmailProgress\" value=\"0\"></progress>");t.b("\n" + i);t.b("                        </button>");t.b("\n" + i);t.b("                    </div>");t.b("\n");t.b("\n" + i);t.b("                    <div class=\"saveDestination\">");t.b("\n" + i);t.b("                        <button id=\"linkSave\">");t.b("\n" + i);t.b("                            <img src=\"/images/link.png\" height=\"25\" width=\"25\">");t.b("\n" + i);t.b("                            <span class=\"destinationName\">Link</span>");t.b("\n" + i);t.b("                            <img class=\"saveCheck\" src=\"/images/check.png\" id=\"saveLinkCheck\"/>");t.b("\n" + i);t.b("                            <progress class=\"buttonProgress\" id=\"saveLinkProgress\" value=\"0\"></progress>");t.b("\n" + i);t.b("                        </button>");t.b("\n" + i);t.b("                    </div>");t.b("\n");t.b("\n" + i);t.b("                    <div class=\"saveDestination\">");t.b("\n" + i);t.b("                        <button id=\"embedSave\">");t.b("\n" + i);t.b("                            <img src=\"//cdn.pizap.com/pizapfiles/images/ic_select_all_grey600_24dp.png\" height=\"25\"");t.b("\n" + i);t.b("                                 width=\"25\">");t.b("\n" + i);t.b("                            <span class=\"destinationName\">Embed</span>");t.b("\n" + i);t.b("                            <img class=\"saveCheck\" src=\"/images/check.png\" id=\"saveEmbedCheck\"/>");t.b("\n" + i);t.b("                            <progress class=\"buttonProgress\" id=\"saveEmbedProgress\" value=\"0\"></progress>");t.b("\n" + i);t.b("                        </button>");t.b("\n" + i);t.b("                    </div>");t.b("\n" + i);t.b("                </div>");t.b("\n" + i);t.b("            </div>");t.b("\n" + i);t.b("            <div class=\"saveDoneBtns\">");t.b("\n" + i);t.b("                <span>Done?</span>");t.b("\n" + i);t.b("                <button id=\"save-done-btn\">Start a New Image</button>");t.b("\n");t.b("\n" + i);t.b("            </div>");t.b("\n" + i);t.b("        </div>");t.b("\n" + i);t.b("    </div>");t.b("\n" + i);t.b("</div>");t.b("\n");t.b("\n" + i);t.b("<div id=\"divEmail\" class=\"shareMenu\" style=\"width:322px;height:310px;z-index:1003;position: absolute; display: none; padding: 0 15px 15px;\" >");t.b("\n" + i);t.b("    <h2>Email this piZap</h2>");t.b("\n" + i);t.b("    <span onclick=\"document.getElementById('divEmail').style.display = 'none';\" style=\"position: absolute; right: 12px; top: 5px; color: #000; cursor: pointer;\">X</span>");t.b("\n" + i);t.b("    <fieldset id=\"inputs\">");t.b("\n" + i);t.b("        <input id=\"recipName\" name=\"recipName\" type=\"text\" placeholder=\"Recipient Name\" autofocus=\"\" required=\"\" autocapitalize=\"off\" autocorrect=\"off\"><br>");t.b("\n" + i);t.b("        <input id=\"recipEmail\" name=\"recipEmail\" type=\"text\" placeholder=\"Recipient Email\" autofocus=\"\" required=\"\" autocapitalize=\"off\" autocorrect=\"off\"><br>");t.b("\n" + i);t.b("        <textarea id=\"taEmailMessage\" placeholder=\"Message\"></textarea>");t.b("\n" + i);t.b("    </fieldset>");t.b("\n" + i);t.b("    <fieldset id=\"actions\">");t.b("\n" + i);t.b("        <input type=\"submit\" id=\"emailSubmit\" value=\"Send Email\" >");t.b("\n" + i);t.b("    </fieldset>");t.b("\n" + i);t.b("</div>");t.b("\n");t.b("\n" + i);t.b("<div id=\"divEmbed\" class=\"shareMenu\" style=\"z-index:1003;width: 410px;height: 250px;position:absolute;text-align:center;display:none;\" align=\"center\">");t.b("\n" + i);t.b("    <h2>Embed this piZap on a blog or website</h2>");t.b("\n" + i);t.b("    <span onclick=\"document.getElementById('divEmbed').style.display = 'none';\" style=\"position: absolute; right: 12px; top: 5px; color: #000; cursor: pointer;\">X</span>");t.b("\n" + i);t.b("    <textarea id='taEmbed' style=\"height:155px;width:390px\" onclick=\"this.select();\"></textarea>");t.b("\n" + i);t.b("</div>");t.b("\n" + i);t.b("<div id=\"divLink\" class=\"shareMenu\" style=\"z-index:1003;width: 410px;height: 250px;position:absolute;text-align:center;display:none;\" align=\"center\">");t.b("\n" + i);t.b("    <h2>Link this piZap on a blog or website</h2>");t.b("\n" + i);t.b("    <span onclick=\"document.getElementById('divLink').style.display = 'none';\" style=\"position: absolute; right: 12px; top: 5px; color: #000; cursor: pointer;\">X</span>");t.b("\n" + i);t.b("    <textarea id='taLink' style=\"height:155px;width:390px\" onclick=\"this.select();\"></textarea>");t.b("\n" + i);t.b("</div>");return t.fl(); },partials: {}, subs: {  }});
}, {"hogan-runtime":25}],
7: [function(require, module, exports) {
var Template = require('hogan-runtime').Template;module.exports = new Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<div id=\"finishLayer\">");t.b("\n" + i);t.b("    <div id=\"finish-dialog\" class=\"photopicker-dialog\">");t.b("\n" + i);t.b("        <div class=\"finish-dialog-box\" style=\"height:220px\">");t.b("\n" + i);t.b("            <div class=\"photopicker-navbar\">");t.b("\n" + i);t.b("                <a id=\"finish-cancel-btn\" class=\"photopicker-navbar-left-btn\"");t.b("\n" + i);t.b("                   style=\"font-weight: lighter;font-size: 14px;\">&lt; Keep Editing</a>");t.b("\n" + i);t.b("            </div>");t.b("\n" + i);t.b("            <div id=\"finishSaveToPizap\" align=\"center\" style=\"padding:30px;font-size:24px\">");t.b("\n" + i);t.b("                <span>Saving Image to piZap...</span>");t.b("\n" + i);t.b("                <progress class=\"buttonProgress\" id=\"saveToPizap\" value=\"0\" max=\".5\"></progress>");t.b("\n" + i);t.b("            </div>");t.b("\n" + i);t.b("            <div id=\"finishButtons\">");t.b("\n" + i);t.b("                <button id=\"newPizap\" onclick=\"window.location.href='/download_complete';\"");t.b("\n" + i);t.b("                        style=\"background-color: #3a7fcc;border-radius: 5px;\">");t.b("\n" + i);t.b("                    <span>Done</span>");t.b("\n" + i);t.b("                </button>");t.b("\n");t.b("\n");t.b("\n" + i);t.b("            </div>");t.b("\n");t.b("\n" + i);t.b("        </div>");t.b("\n" + i);t.b("    </div>");t.b("\n" + i);t.b("</div>");return t.fl(); },partials: {}, subs: {  }});
}, {"hogan-runtime":25}],
8: [function(require, module, exports) {
var Template = require('hogan-runtime').Template;module.exports = new Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<div class=\"photopicker-list-item\" data-id=\"");t.b(t.v(t.f("id",c,p,0)));t.b("\">");t.b("\n" + i);t.b("    <img src=\"");t.b(t.v(t.f("icon",c,p,0)));t.b("\"/>");t.b("\n" + i);t.b("    <span class=\"name\">");t.b(t.v(t.f("name",c,p,0)));t.b("</span>");t.b("\n" + i);t.b("</div>");return t.fl(); },partials: {}, subs: {  }});
}, {"hogan-runtime":25}],
9: [function(require, module, exports) {
var Template = require('hogan-runtime').Template;module.exports = new Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<div class=\"photopicker-photo-item\" data-id=\"");t.b(t.v(t.f("id",c,p,0)));t.b("\">");t.b("\n" + i);t.b("    <img src=\"");t.b(t.v(t.f("picture",c,p,0)));t.b("\" data-src=\"");t.b(t.v(t.f("source",c,p,0)));t.b("\"/>");t.b("\n" + i);t.b("</div>");return t.fl(); },partials: {}, subs: {  }});
}, {"hogan-runtime":25}],
10: [function(require, module, exports) {
var Template = require('hogan-runtime').Template;module.exports = new Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<div id=\"menu-meme\">");t.b("\n" + i);t.b("    <div id=\"menu-meme-top\">");t.b("\n" + i);t.b("        <span style=\"display:block\">Top Text</span>");t.b("\n" + i);t.b("        <input id=\"memeTopText\" name=\"memeTopText\" type=\"text\">");t.b("\n" + i);t.b("        <span id=\"memeTopPlus\" class=\"memePlusMinus\">+</span>");t.b("\n" + i);t.b("        <span id=\"memeTopMinus\" class=\"memePlusMinus\">-</span>");t.b("\n" + i);t.b("        <div class=\"preview\" id=\"previewTop\"></div>");t.b("\n" + i);t.b("    </div>");t.b("\n" + i);t.b("    <div id=\"menu-meme-bottom\">");t.b("\n" + i);t.b("        <span style=\"display:block\">Bottom Text</span>");t.b("\n" + i);t.b("        <input id=\"memeBottomText\" name=\"memeBottomText\" type=\"text\">");t.b("\n" + i);t.b("        <span id=\"memeBottomPlus\" class=\"memePlusMinus\">+</span>");t.b("\n" + i);t.b("        <span id=\"memeBottomMinus\" class=\"memePlusMinus\">-</span>");t.b("\n" + i);t.b("        <div class=\"preview\" id=\"previewBottom\"></div>");t.b("\n" + i);t.b("    </div>");t.b("\n" + i);t.b("</div>");t.b("\n");return t.fl(); },partials: {}, subs: {  }});
}, {"hogan-runtime":25}],
11: [function(require, module, exports) {
"use strict";


var inited = false;
var parentCallback = null;
var multiselect = false;
var loadingTemplate = require('./templates/loading.hogan');

function downloadFile(downloadUrl, callback) {
    var xhr = new XMLHttpRequest();
    xhr.open('GET', downloadUrl);
    if ( xhr.overrideMimeType ) xhr.overrideMimeType("text/plain; charset=x-user-defined");
    xhr.onload = function() {
        callback(xhr.responseText);
    };
    xhr.onerror = function() {
        callback(null);
    };
    xhr.send();
}

function ajaxJSONGet(url, callback){
    var http_request = new XMLHttpRequest();
    http_request.open("GET", url, true);
    http_request.onreadystatechange = function () {
        var done = 4;
        var ok = 200;
        if (http_request.readyState === done && http_request.status === ok){
            callback(JSON.parse(http_request.responseText));
        }
    };
    http_request.send();
}

function waitFor(handler, next, timeout) {
    timeout = timeout || 100;
    var result = handler ? handler(): true;

    if (! result) {
        setTimeout(function() {
            waitFor(handler, next, timeout);
        }, timeout);
    } else {
        next && next();
    }
}

function base64Encode(str) {
    var CHARS = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
    var out = "", i = 0, len = str.length, c1, c2, c3;
    while (i < len) {
        c1 = str.charCodeAt(i++) & 0xff;
        if (i == len) {
            out += CHARS.charAt(c1 >> 2);
            out += CHARS.charAt((c1 & 0x3) << 4);
            out += "==";
            break;
        }
        c2 = str.charCodeAt(i++);
        if (i == len) {
            out += CHARS.charAt(c1 >> 2);
            out += CHARS.charAt(((c1 & 0x3)<< 4) | ((c2 & 0xF0) >> 4));
            out += CHARS.charAt((c2 & 0xF) << 2);
            out += "=";
            break;
        }
        c3 = str.charCodeAt(i++);
        out += CHARS.charAt(c1 >> 2);
        out += CHARS.charAt(((c1 & 0x3) << 4) | ((c2 & 0xF0) >> 4));
        out += CHARS.charAt(((c2 & 0xF) << 2) | ((c3 & 0xC0) >> 6));
        out += CHARS.charAt(c3 & 0x3F);
    }
    return out;
}

function openPickDialog() {
    var html = loadingTemplate.render();

    var bodyEl = document.getElementsByTagName('BODY')[0];
    var d = document.getElementById("waiting");
    if(d){
        bodyEl.removeChild(d);
    }
    bodyEl.insertAdjacentHTML('beforeend', html);


    var fileSelect = document.getElementById("s_file");
    if(multiselect){
        fileSelect = document.getElementById("s_files");
    }
    fileSelect.addEventListener('change', function _func(event) {
        event.target.removeEventListener(event.type, _func);
        photoOnSelected(event);
    });
    fileSelect.click();
}


function photoOnSelected(event) {
    var target = event.target;
    var result = [];

    activate('dialog');

    if (target !== null) {
        event.preventDefault();
        event.stopPropagation();

        var fileSelect = document.getElementById("s_files");
        if(multiselect){
            if (fileSelect && fileSelect.files && fileSelect.files.length > 0) {
                for(var i = 0; i < fileSelect.files.length; i++) {
                    (function(file) {
                        var reader = new FileReader();
                        reader.onloadend = function () {
                            result.push(reader.result);
                        }
                        if (file) {
                            reader.readAsDataURL(file);
                        }
                    })(fileSelect.files[i]);
                }
            }

        }else {
            fileSelect = document.getElementById("s_file");
            if (fileSelect && fileSelect.files && fileSelect.files.length > 0) {
                var file = fileSelect.files[0];
                var reader = new FileReader();

                reader.onloadend = function () {
                    result.push(reader.result);
                }

                if (file) {
                    reader.readAsDataURL(file);
                }
            }
        }
        waitFor(function() {
            return fileSelect.files.length === result.length;
        }, function() {
            fileSelect.value = "";

            parentCallback && parentCallback(result);
            deactivate('dialog');
        });
    } else {
        parentCallback && parentCallback(null);
        deactivate('dialog');
    }
}

function activate(id) {
    var el = document.getElementById('waiting-' + id);

    if (el) {
        el.classList.add('active');
    }
}

function deactivate(id) {
    var el = document.getElementById('waiting-' + id);

    if (el) {
        el.classList.remove('active');
    }
}

// Exports

module.exports = function(options, callback) {

    if (typeof callback !== undefined) {
        parentCallback = callback;
    }

    if (typeof options.multiselect !== 'undefined') {
        multiselect = !! options.multiselect;
    }

    if (! inited) {
        inited = true;
    }

    openPickDialog();
};
}, {"./templates/loading.hogan":26}],
26: [function(require, module, exports) {
var Template = require('hogan-runtime').Template;module.exports = new Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<div class=\"waiting\">");t.b("\n" + i);t.b("    <div id=\"waiting-dialog\" class=\"waiting-dialog\">");t.b("\n" + i);t.b("        <div class=\"waiting-dialog-box\">");t.b("\n" + i);t.b("            <div class=\"waiting-navbar\">");t.b("\n" + i);t.b("                <div class=\"filepicker-title\">Loading...</div>");t.b("\n" + i);t.b("            </div>");t.b("\n" + i);t.b("            <img src=\"//cdn.pizap.com/pizapfiles/images/loading_spinner.gif\" width=\"30\" height=\"30\"/><br/>");t.b("\n" + i);t.b("        </div>");t.b("\n" + i);t.b("    </div>");t.b("\n" + i);t.b("</div>");return t.fl(); },partials: {}, subs: {  }});
}, {"hogan-runtime":25}],
12: [function(require, module, exports) {
"use strict";


// Varibles

var clientId;
var redirectUri;
var inited = false;
var parentCallback = null;
var multiselect = false;
var dialogTemplate = require('./templates/file-picker.hogan');
var sourceItemTemplate = require('./templates/album-item.hogan');
var photoItemTemplate = require('./templates/photo-item.hogan');
var loadingTemplate = require('./templates/loading.hogan');

// Functions
function waitingActivate(id) {
    var el = document.getElementById('waiting-' + id);

    if (el) {
        el.classList.add('active');
    }
}

function waitingDeactivate(id) {
    var el = document.getElementById('waiting-' + id);

    if (el) {
        el.classList.remove('active');
    }
}
function downloadFile(downloadUrl, callback) {
    var xhr = new XMLHttpRequest();
    xhr.open('GET', downloadUrl);
    if ( xhr.overrideMimeType ) xhr.overrideMimeType("text/plain; charset=x-user-defined");
    xhr.onload = function() {
        callback(xhr.responseText);
    };
    xhr.onerror = function() {
        callback(null);
    };
    xhr.send();
}

function waitFor(handler, next, timeout) {
    timeout = timeout || 100;
    var result = handler ? handler(): true;

    if (! result) {
        setTimeout(function() {
            waitFor(handler, next, timeout);
        }, timeout);
    } else {
        next && next();
    }
}

function base64Encode(str) {
    var CHARS = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
    var out = "", i = 0, len = str.length, c1, c2, c3;
    while (i < len) {
        c1 = str.charCodeAt(i++) & 0xff;
        if (i == len) {
            out += CHARS.charAt(c1 >> 2);
            out += CHARS.charAt((c1 & 0x3) << 4);
            out += "==";
            break;
        }
        c2 = str.charCodeAt(i++);
        if (i == len) {
            out += CHARS.charAt(c1 >> 2);
            out += CHARS.charAt(((c1 & 0x3)<< 4) | ((c2 & 0xF0) >> 4));
            out += CHARS.charAt((c2 & 0xF) << 2);
            out += "=";
            break;
        }
        c3 = str.charCodeAt(i++);
        out += CHARS.charAt(c1 >> 2);
        out += CHARS.charAt(((c1 & 0x3) << 4) | ((c2 & 0xF0) >> 4));
        out += CHARS.charAt(((c2 & 0xF) << 2) | ((c3 & 0xC0) >> 6));
        out += CHARS.charAt(c3 & 0x3F);
    }
    return out;
}

function openPickDialog() {
    var bodyEl = document.getElementsByTagName('BODY')[0];
    var waitingHtml = loadingTemplate.render();
    var w = document.getElementById("waiting");
    if(w){
        bodyEl.removeChild(w);
    }
    bodyEl.insertAdjacentHTML('beforeend', waitingHtml);
    window.FB.getLoginStatus(function(response) {
        if (response.status === 'connected') {
            FB.api('/me/permissions', function(response){
                console.log("permissions returned");
                var found_perm = false;
                if(response && response.data){
                    for (var i = 0; i < response.data.length; i++) {
                        if(response.data[i].permission == "user_photos"){
                            found_perm = true;
                        }
                    }
                }
                if(!found_perm){
                    FB.login(function(response) {getFbAlbums();}, {scope: 'email,user_photos'});
                }else{
                    getFbAlbums();
                }
            });
        } else if (response.status === 'not_authorized') {
            console.log("not authorized");
            FB.login(function(response) {getFbAlbums();}, {scope: 'email,user_photos'});
        } else {
            console.log("unkown status" + response.status);
            FB.login(function(response) {getFbAlbums();}, {scope: 'email,user_photos'});
        }
    });
}

function getFbAlbums(){
    initDialog();
    console.log("get facebook albums");
    waitingActivate('dialog');
    FB.api("/me/albums", "get", null, function(response){
        console.log("get facebook albums returned");
        if(response && response.data){
            var listEl = document.getElementById('photopicker-albums-list');
            while (listEl.firstChild) {
                listEl.removeChild(listEl.firstChild);
            }
            for(var i = 0; i < response.data.length;i++){
                if(i == 0){
                    getFacebookPhotos(response.data[i].id, function() {
                        deactivate('dialog-loading');
                    });
                }
                listEl.insertAdjacentHTML('beforeend', sourceItemTemplate.render(response.data[i]));
            }
            waitingDeactivate('dialog');
            activate('dialog');
        }
        console.log("created new result of albums");
    });
}
function findParentNode(node, attrName) {
    if (! node) {
        return null;
    }

    while (node) {
        if (node.getAttribute && node.getAttribute(attrName) !== null) {
            break;
        }
        node = node.parentNode;
    }

    return node;
}
function initDialog() {
    var html = dialogTemplate.render();

    var bodyEl = document.getElementsByTagName('BODY')[0];
    var d = document.getElementById("photopicker");
    if(d){
        bodyEl.removeChild(d);
    }

    bodyEl.insertAdjacentHTML('beforeend', html);

    var cancelBtn = document.getElementById('photopicker-cancel-btn');
    cancelBtn.addEventListener('click', function(event) {
        event.preventDefault();
        event.stopPropagation();

        var listEl = document.getElementById('photopicker-albums-list');
        listEl.removeEventListener('click', albumOnSelected);

        var listEl = document.getElementById('photopicker-album-files');
        listEl.removeEventListener('click', photoOnSelected);

        deactivate('dialog');

        //callback && callback(null);
    }, false);

    var listEl = document.getElementById('photopicker-albums-list');
    listEl.removeEventListener('click', albumOnSelected);
    listEl.addEventListener('click', albumOnSelected);

    var listEl = document.getElementById('photopicker-album-files');
    listEl.removeEventListener('click', photoOnSelected);
    listEl.addEventListener('click', photoOnSelected);
    document.getElementById('photopicker-album-files').addEventListener("scroll", function (event) {
        checkForNewDiv();
    });

    var listEls = document.getElementById('scroll-content');
    while (listEls.firstChild) {
        listEls.removeChild(listEls.firstChild);
    }


    var checkForNewDiv = function() {
        if(loading == true) return;
        var wrapper = document.getElementById('photopicker-album-files');
        var lastDiv = document.querySelector("#scroll-content > div:last-child");
        var lastDivOffset = lastDiv.offsetTop + lastDiv.clientHeight;
        var pageOffset = wrapper.scrollTop + wrapper.clientHeight;

        if (pageOffset > lastDivOffset - 240) {
            loading = true;
            getFacebookPhotos(currentId, currentCb, false)
        }
    };
}
function albumOnSelected(event) {
    var target = findParentNode(event.target, 'data-id');

    if (target !== null) {
        event.preventDefault();
        event.stopPropagation();

        var sourceId = target.getAttribute('data-id');
        activate('dialog-loading');
        afterCursor = null;
        getFacebookPhotos(sourceId, function() {
            deactivate('dialog-loading');
        });
    }
}

function photoOnSelected(event) {
    var target = event.target;
    var result = [];

    if (target !== null && target.getAttribute('data-src')) {
        waitingActivate('dialog');
        event.preventDefault();
        event.stopPropagation();

        var listEl = document.getElementById('photopicker-albums-list');
        listEl.removeEventListener('click', albumOnSelected);

        var listEl = document.getElementById('photopicker-album-files');
        listEl.removeEventListener('click', photoOnSelected);

        var source = target.getAttribute('data-src');
        afterCursor = null;
        console.log("photo selected");
        console.log(source);
        deactivate('dialog');

        downloadFile(source, (function(file) {
            var ext = file.toLowerCase().match(/\.(jpg|jpeg|png|gif)/);
            ext = typeof ext[1] !== 'undefined' ? ext[1]: null;
            var mimeType = null;

            switch (ext) {
                case 'png': mimeType = 'image/png'; break;
                case 'gif': mimeType = 'image/gif'; break;
                case 'jpg':
                case 'jpeg':
                default: mimeType = 'image/jpeg'; break;
            }

            return function(content) {
                setTimeout(function() {
                    result.push('data:' + mimeType + ';base64,' + base64Encode(content));
                }, 1);
            };
        })(source));

        waitFor(function() {
            return 1 === result.length;
        }, function() {
            waitingDeactivate('dialog');
            parentCallback && parentCallback(result);
        });
    } else {
        waitingDeactivate('dialog');
        parentCallback && parentCallback(null);
    }
}

var afterCursor = null;
var currentId = null;
var currentCb = null;
var loading = false;

function getFacebookPhotos(id, cb, removeOld) {
    currentId = id;
    currentCb = cb;
    FB.api(
        "/" + id + "/photos?fields=images,picture", {after: afterCursor},
        function (response) {
            loading = false;
            if (response && !response.error) {
                var listEl = document.getElementById('scroll-content');
                if(removeOld != false) {
                    while (listEl.firstChild) {
                        listEl.removeChild(listEl.firstChild);
                    }
                }
                for(var i = 0; i < response.data.length;i++){
                    var item = {id: response.data[i].id, picture: response.data[i].picture, source: response.data[i].images[0].source};
                    listEl.insertAdjacentHTML('beforeend', photoItemTemplate.render(item));
                }
                listEl.style.webkitTransform = 'scale(1)';
                if(response.data.length > 0) {
                    if (response.paging && response.paging.cursors && response.paging.cursors.after)
                        afterCursor = response.paging.cursors.after;
                    loading =  false;
                }else{
                    loading = true;
                }
                cb && cb();
            }
        }
    );

}

function activate(id) {
    var el = document.getElementById('photopicker-' + id);

    if (el) {
        el.classList.add('active');
    }
}

function deactivate(id) {
    var el = document.getElementById('photopicker-' + id);

    if (el) {
        el.classList.remove('active');
    }
}

// Exports

module.exports = function(options, callback) {
    if (typeof window.FB === 'undefined') {
        alert('Facebook API is not loaded.');
        return;
    }

    if (typeof callback !== undefined) {
        parentCallback = callback;
    }

    if (typeof options.multiselect !== 'undefined') {
        multiselect = !! options.multiselect;
    }

    if (! inited) {

        inited = true;
    }

    openPickDialog();
};
}, {"./templates/file-picker.hogan":27,"./templates/album-item.hogan":28,"./templates/photo-item.hogan":9,"./templates/loading.hogan":26}],
27: [function(require, module, exports) {
var Template = require('hogan-runtime').Template;module.exports = new Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<div class=\"photopicker\" id=\"photopicker\">");t.b("\n" + i);t.b("    <div id=\"photopicker-dialog\" class=\"photopicker-dialog\">");t.b("\n" + i);t.b("        <div class=\"photopicker-dialog-box\">");t.b("\n" + i);t.b("            <div class=\"photopicker-navbar\">");t.b("\n" + i);t.b("                <div class=\"photopicker-title\">Select a photo</div>");t.b("\n" + i);t.b("                <a id=\"photopicker-cancel-btn\" class=\"photopicker-navbar-right-btn\">Cancel</a>");t.b("\n" + i);t.b("            </div>");t.b("\n");t.b("\n" + i);t.b("            <div id=\"photopicker-container\">");t.b("\n" + i);t.b("                <div id=\"photopicker-albums-list\" class=\"photopicker-list\"></div>");t.b("\n" + i);t.b("                <div id=\"photopicker-album-files\" class=\"photopicker-photos\">");t.b("\n" + i);t.b("                    <div id=\"scroll-content\">");t.b("\n");t.b("\n" + i);t.b("                    </div>");t.b("\n" + i);t.b("                </div>");t.b("\n" + i);t.b("            </div>");t.b("\n" + i);t.b("            <div id=\"photopicker-dialog-loading\" class=\"photopicker-loading\">Loading&hellip;</div>");t.b("\n" + i);t.b("        </div>");t.b("\n" + i);t.b("    </div>");t.b("\n" + i);t.b("</div>");return t.fl(); },partials: {}, subs: {  }});
}, {"hogan-runtime":25}],
28: [function(require, module, exports) {
var Template = require('hogan-runtime').Template;module.exports = new Template({code: function (c,p,i) { var t=this;t.b(i=i||"");t.b("<div class=\"photopicker-list-item\" data-id=\"");t.b(t.v(t.f("id",c,p,0)));t.b("\">");t.b("\n" + i);t.b("    <span class=\"name\">");t.b(t.v(t.f("name",c,p,0)));t.b("</span>");t.b("\n" + i);t.b("</div>");return t.fl(); },partials: {}, subs: {  }});
}, {"hogan-runtime":25}],
13: [function(require, module, exports) {
"use strict";

var loadingTemplate = require('./templates/loading.hogan');

// Functions
function waitingActivate(id) {
    var el = document.getElementById('waiting-' + id);

    if (el) {
        el.classList.add('active');
    }
}

function waitingDeactivate(id) {
    var el = document.getElementById('waiting-' + id);

    if (el) {
        el.classList.remove('active');
    }
}
function downloadFile(downloadUrl, callback) {
  var xhr = new XMLHttpRequest();
  xhr.open('GET', downloadUrl);
    if ( xhr.overrideMimeType ) xhr.overrideMimeType("text/plain; charset=x-user-defined");
  xhr.onload = function() {
    callback(xhr.responseText);
  };
  xhr.onerror = function() {
    callback(null);
  };
  xhr.send();
}

function waitFor(handler, next, timeout) {
  timeout = timeout || 100;
  var result = handler ? handler(): true;

  if (! result) {
    setTimeout(function() {
      waitFor(handler, next, timeout);
    }, timeout);
  } else {
    next && next();
  }
}

function base64Encode(str) {
    var CHARS = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
    var out = "", i = 0, len = str.length, c1, c2, c3;
    while (i < len) {
        c1 = str.charCodeAt(i++) & 0xff;
        if (i == len) {
            out += CHARS.charAt(c1 >> 2);
            out += CHARS.charAt((c1 & 0x3) << 4);
            out += "==";
            break;
        }
        c2 = str.charCodeAt(i++);
        if (i == len) {
            out += CHARS.charAt(c1 >> 2);
            out += CHARS.charAt(((c1 & 0x3)<< 4) | ((c2 & 0xF0) >> 4));
            out += CHARS.charAt((c2 & 0xF) << 2);
            out += "=";
            break;
        }
        c3 = str.charCodeAt(i++);
        out += CHARS.charAt(c1 >> 2);
        out += CHARS.charAt(((c1 & 0x3) << 4) | ((c2 & 0xF0) >> 4));
        out += CHARS.charAt(((c2 & 0xF) << 2) | ((c3 & 0xC0) >> 6));
        out += CHARS.charAt(c3 & 0x3F);
    }
    return out;
}


// Exports

module.exports = function(options, callback) {
  if (typeof window.Dropbox === 'undefined') {
    alert('Dropbox API is not loaded.');
    return;
  }

  var multiselect = !! options.multiselect;

  var dropboxОptions = {
    // Required. Called when a user selects an item in the Chooser.
    success: function(files) {
      var result = null;
      var fileUrl;
        var bodyEl = document.getElementsByTagName('BODY')[0];
        var waitingHtml = loadingTemplate.render();
        var w = document.getElementById("waiting");
        if(w){
            bodyEl.removeChild(w);
        }
        bodyEl.insertAdjacentHTML('beforeend', waitingHtml);

      if (files.length > 0) {
          waitingActivate('dialog');
          result = [];

        var size = multiselect ? 800: 1280;

        for (var i=0,len=files.length; i<len; i++) {
          fileUrl = files[i].thumbnailLink.replace('bounding_box=75', 'bounding_box=' + size);

          downloadFile(fileUrl, (function(file) {
            var ext = file.name.toLowerCase().match(/\.(jpg|jpeg|png|gif)$/);
            ext = typeof ext[1] !== 'undefined' ? ext[1]: null;
            var mimeType = null;

            switch (ext) {
              case 'png': mimeType = 'image/png'; break;
              case 'gif': mimeType = 'image/gif'; break;
              case 'jpg':
              case 'jpeg':
              default: mimeType = 'image/jpeg'; break;
            }

            return function(content) {
              setTimeout(function() {
                result.push('data:' + mimeType + ';base64,' + base64Encode(content));
              }, 1);
            };
          })(files[i]));
        }

        waitFor(function() {
          return files.length === result.length;
        }, function() {
            waitingDeactivate('dialog');
            callback && callback(result);
        });
      } else {
          waitingDeactivate('dialog');
          callback && callback(result);
      }
    },

    // Optional. Called when the user closes the dialog without selecting a file
    // and does not include any parameters.
    cancel: function() {
      callback && callback(null);
    },

    // Optional. "preview" (default) is a preview link to the document for sharing,
    // "direct" is an expiring link to download the contents of the file. For more
    // information about link types, see Link types below.
    linkType: "direct", // or "direct"

    // Optional. A value of false (default) limits selection to a single file, while
    // true enables multiple file selection.
    multiselect: false, // or true

    // Optional. This is a list of file extensions. If specified, the user will
    // only be able to select files with these extensions. You may also specify
    // file types, such as "video" or "images" in the list. For more information,
    // see File types below. By default, all extensions are allowed.
    extensions: ['.gif', '.jpg', '.jpeg', '.png'],
  };

  if (typeof options.multiselect !== 'undefined') {
    dropboxОptions.multiselect = multiselect;
  }

  try {
    window.Dropbox.choose(dropboxОptions);
  } catch (e) {
    // alert('Dropbox choose error.');
    callback && callback(null);
  }
};
}, {"./templates/loading.hogan":26}],
14: [function(require, module, exports) {
"use strict";


// Varibles

var clientId;
var redirectUri;
var inited = false;
var parentCallback = null;
var multiselect = false;
var loadingTemplate = require('./templates/loading.hogan');

// Functions
function waitingActivate(id) {
    var el = document.getElementById('waiting-' + id);

    if (el) {
        el.classList.add('active');
    }
}

function waitingDeactivate(id) {
    var el = document.getElementById('waiting-' + id);

    if (el) {
        el.classList.remove('active');
    }
}
function downloadFile(downloadUrl, callback) {
  var xhr = new XMLHttpRequest();
  xhr.open('GET', downloadUrl);
    if ( xhr.overrideMimeType ) xhr.overrideMimeType("text/plain; charset=x-user-defined");
  xhr.onload = function() {
    callback(xhr.responseText);
  };
  xhr.onerror = function() {
    callback(null);
  };
  xhr.send();
}

function waitFor(handler, next, timeout) {
  timeout = timeout || 100;
  var result = handler ? handler(): true;

  if (! result) {
    setTimeout(function() {
      waitFor(handler, next, timeout);
    }, timeout);
  } else {
    next && next();
  }
}

function base64Encode(str) {
    var CHARS = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
    var out = "", i = 0, len = str.length, c1, c2, c3;
    while (i < len) {
        c1 = str.charCodeAt(i++) & 0xff;
        if (i == len) {
            out += CHARS.charAt(c1 >> 2);
            out += CHARS.charAt((c1 & 0x3) << 4);
            out += "==";
            break;
        }
        c2 = str.charCodeAt(i++);
        if (i == len) {
            out += CHARS.charAt(c1 >> 2);
            out += CHARS.charAt(((c1 & 0x3)<< 4) | ((c2 & 0xF0) >> 4));
            out += CHARS.charAt((c2 & 0xF) << 2);
            out += "=";
            break;
        }
        c3 = str.charCodeAt(i++);
        out += CHARS.charAt(c1 >> 2);
        out += CHARS.charAt(((c1 & 0x3) << 4) | ((c2 & 0xF0) >> 4));
        out += CHARS.charAt(((c2 & 0xF) << 2) | ((c3 & 0xC0) >> 6));
        out += CHARS.charAt(c3 & 0x3F);
    }
    return out;
}

function openPickDialog() {
    var bodyEl = document.getElementsByTagName('BODY')[0];
    var waitingHtml = loadingTemplate.render();
    var w = document.getElementById("waiting");
    if(w){
        bodyEl.removeChild(w);
    }
    bodyEl.insertAdjacentHTML('beforeend', waitingHtml);

  window.WL.fileDialog({
    mode: 'open',
    select: multiselect ? 'multi': 'single'
  }).then(
      function (response) {
          var result = null;
          var files = [];

          if (response.data.files.length > 0) {
            files = [];
            var item;

            for (var i=0,len=response.data.files.length; i<len; i++) {
              item = response.data.files[i];

              if (item.type != 'photo') {
                alert('Only images are allowed. Please, select only images.');
                openPickDialog();
                return;
              }

              files.push(item);
            }
          }

          if (files.length > 0) {
              waitingActivate('dialog');
              result = [];
            var fileUrl;
            var accessToken = window.WL.getSession().access_token;

            for (var i=0,len=files.length; i<len; i++) {
              fileUrl = 'https://apis.live.net/v5.0/'+files[i].id+'/picture?type='+(multiselect ? 'normal': 'full')+'&access_token='+accessToken;

              downloadFile(fileUrl, (function(file) {
                var ext = file.name.toLowerCase().match(/\.(jpg|jpeg|png|gif)$/);
                ext = typeof ext[1] !== 'undefined' ? ext[1]: null;
                var mimeType = null;

                switch (ext) {
                  case 'png': mimeType = 'image/png'; break;
                  case 'gif': mimeType = 'image/gif'; break;
                  case 'jpg':
                  case 'jpeg':
                  default: mimeType = 'image/jpeg'; break;
                }

                return function(content) {
                  setTimeout(function() {
                    result.push('data:' + mimeType + ';base64,' + base64Encode(content));
                  }, 1);
                };
              })(files[i]));
            }

            waitFor(function() {
              return files.length === result.length;
            }, function() {
                waitingDeactivate('dialog');
                parentCallback && parentCallback(result);
            });
          } else {
              waitingDeactivate('dialog');
              parentCallback && parentCallback(null);
          }
      },
      function (responseFailed) {
        parentCallback && parentCallback(null);
      }
  );
}


// Exports

module.exports = function(options, callback) {
  if (typeof window.WL === 'undefined') {
    alert('OneDrive API is not loaded.');
    return;
  }

  if (typeof options.clientId !== 'undefined') {
    clientId = options.clientId;
  }

  if (typeof options.redirectUri !== 'undefined') {
    redirectUri = options.redirectUri;
  }

  if (typeof callback !== undefined) {
    parentCallback = callback;
  }

  if (typeof options.multiselect !== 'undefined') {
    multiselect = !! options.multiselect;
  }

  if (! inited) {
    window.WL.init({
      client_id: clientId,
      redirect_uri: redirectUri,
      scope: 'wl.signin', 
      response_type: 'token'
    });

    inited = true;
  }

  openPickDialog();
};
}, {"./templates/loading.hogan":26}],
15: [function(require, module, exports) {
"use strict";

function base64Encode(str) {
    var CHARS = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
    var out = "", i = 0, len = str.length, c1, c2, c3;
    while (i < len) {
        c1 = str.charCodeAt(i++) & 0xff;
        if (i == len) {
            out += CHARS.charAt(c1 >> 2);
            out += CHARS.charAt((c1 & 0x3) << 4);
            out += "==";
            break;
        }
        c2 = str.charCodeAt(i++);
        if (i == len) {
            out += CHARS.charAt(c1 >> 2);
            out += CHARS.charAt(((c1 & 0x3)<< 4) | ((c2 & 0xF0) >> 4));
            out += CHARS.charAt((c2 & 0xF) << 2);
            out += "=";
            break;
        }
        c3 = str.charCodeAt(i++);
        out += CHARS.charAt(c1 >> 2);
        out += CHARS.charAt(((c1 & 0x3) << 4) | ((c2 & 0xF0) >> 4));
        out += CHARS.charAt(((c2 & 0xF) << 2) | ((c3 & 0xC0) >> 6));
        out += CHARS.charAt(c3 & 0x3F);
    }
    return out;
}

// Varibles

var developerKey; // The Browser API key obtained from the Google Developers Console.
var clientId; // The Client ID obtained from the Google Developers Console.
var scope = ['https://www.googleapis.com/auth/photos']; // Scope to use to access user's photos.
var pickerApiLoaded = false;
var pickerAuthLoaded = false;
var oauthToken = null;
var parentCallback;
var isLoaded = false;
var multiselect = false;
var loadingTemplate = require('./templates/loading.hogan');

window['sourceLoaded-google-drive'] = function sourceLoadedHandler() {
  window.gapi.load('auth');

  window.gapi.load('picker', {
    callback: onPickerApiLoad
  });

  window.gapi.load('client');
};

function waitingActivate(id) {
  var el = document.getElementById('waiting-' + id);

  if (el) {
    el.classList.add('active');
  }
}

function waitingDeactivate(id) {
  var el = document.getElementById('waiting-' + id);

  if (el) {
    el.classList.remove('active');
  }
}
// Functions

// wait untill handler function return true
function waitFor(handler, next, timeout) {
  timeout = timeout || 100;
  var result = handler ? handler(): true;

  if (! result) {
    setTimeout(function() {
      waitFor(handler, next, timeout);
    }, timeout);
  } else {
    next && next();
  }
}

function onAuthApiLoad() {
  pickerAuthLoaded = true;
}

function onPickerApiLoad() {
  pickerApiLoaded = true;
}

function handleAuthResult(authResult) {
  if (authResult && !authResult.error) {
    oauthToken = authResult.access_token;
    createPicker();
  }
}

// Create and render a Picker object for picking user Photos.
function createPicker() {
  // console.log('pickerApiLoaded && oauthToken', pickerApiLoaded, oauthToken);
  if (pickerApiLoaded && oauthToken) {
    var bodyEl = document.getElementsByTagName('BODY')[0];
    var waitingHtml = loadingTemplate.render();
    var w = document.getElementById("waiting");
    if(w){
      bodyEl.removeChild(w);
    }
    bodyEl.insertAdjacentHTML('beforeend', waitingHtml);
    var view = new google.picker.DocsView()
      .setMimeTypes('image/jpeg,image/png,image/gif')
      .setParent('root')
      .setIncludeFolders(true);

    var picker = new google.picker.PickerBuilder().
      setRelayUrl(document.location.protocol + '//' + document.location.hostname + (document.location.port ? ':' + document.location.port: '')).
      addView(view). // google.picker.ViewId.DOCS_IMAGES
      setOAuthToken(oauthToken).
      setDeveloperKey(developerKey).
      setCallback(pickerCallback);

    if (multiselect) {
      picker.enableFeature(google.picker.Feature.MULTISELECT_ENABLED);
    }

    picker = picker.build();

    picker.setVisible(true);
  }
}

// A simple callback implementation.
function pickerCallback(data) {
  if (data[google.picker.Response.ACTION] == google.picker.Action.PICKED) {
    var docs = data[google.picker.Response.DOCUMENTS];
    var result = null;

    if (docs.length > 0) {
      waitingActivate('dialog');
      result = [];
      for (var i=0,len=docs.length; i<len; i++) {
        if (docs[i].mimeType !== 'image/jpeg'
          && docs[i].mimeType !== 'image/gif'
          && docs[i].mimeType !== 'image/png') {
          alert('Only images are allowed. Please, select only images.');
          result = null;
          createPicker();
          break;
        }

        getDownloadUrl(docs[i].id, function(url) {
          result.push(url);
        });
      }
    }

    waitFor(function() {
      if (result === null) {
        return true;
      }

      return result.length === docs.length;
    }, function() {
      waitingDeactivate('dialog');
      parentCallback && parentCallback(result);
    })
  } else if (data[google.picker.Response.ACTION] == google.picker.Action.CANCEL) {
    waitingDeactivate('dialog');
    parentCallback && parentCallback(null);
  }
}

function downloadFile(url, callback) {
  var accessToken = gapi.auth.getToken().access_token;
  var xhr = new XMLHttpRequest();
  xhr.open('GET', url);
  if ( xhr.overrideMimeType ) xhr.overrideMimeType("text/plain; charset=x-user-defined");
  // xhr.setRequestHeader('Authorization', 'Bearer ' + accessToken);
  xhr.onload = function() {
    callback(xhr.responseText);
  };
  xhr.onerror = function() {
    callback(null);
  };
  xhr.send();
}

function getDownloadUrl(fileId, cb) {
  var request = gapi.client.request({
    'path': '/drive/v2/files/' + fileId,
    'method': 'GET',
    'params': {'maxResults': '1'},
    callback: function (responsejs, responsetxt) {
      var fileUrl = responsejs.thumbnailLink.replace('s220', multiselect ? 's800': 's1280');
      console.log(fileUrl);
      downloadFile(fileUrl, function(content) {
        typeof cb === 'function' && cb('data:' + responsejs.mimeType + ';base64,' + base64Encode(content));
      });
    }
  });
}


// Exports

module.exports = function(options, callback) {
  if (typeof window.gapi === 'undefined') {
    alert('Google API is not loaded.');
    parentCallback && parentCallback(null);
    return;
  }

  if (typeof options.scope !== 'undefined') {
    scope = options.scope;
  }

  if (typeof options.developerKey !== 'undefined') {
    developerKey = options.developerKey;
  }

  if (typeof options.clientId !== 'undefined') {
    clientId = options.clientId;
  }

  if (typeof options.oauthToken !== 'undefined') {
    oauthToken = options.oauthToken;
  }

  if (typeof options.multiselect !== 'undefined') {
    multiselect = !! options.multiselect;
  }

  parentCallback = callback;

  if (! oauthToken) {
    window.gapi.auth.authorize(
      {
        client_id: clientId,
        scope: scope,
        immediate: false
      },
      handleAuthResult
    );
  } else {
    createPicker();
  }
};
}, {"./templates/loading.hogan":26}],
16: [function(require, module, exports) {
"use strict";


// Varibles

var clientId;
var redirectUri;
var inited = false;
var parentCallback = null;
var multiselect = false;
var dialogTemplate = require('./templates/file-picker.hogan');
var sourceItemTemplate = require('./templates/album-item.hogan');
var photoItemTemplate = require('./templates/photo-item.hogan');
var username;
var loadingTemplate = require('./templates/loading.hogan');
// Functions
function waitingActivate(id) {
  var el = document.getElementById('waiting-' + id);

  if (el) {
    el.classList.add('active');
  }
}

function waitingDeactivate(id) {
  var el = document.getElementById('waiting-' + id);

  if (el) {
    el.classList.remove('active');
  }
}
function downloadFile(downloadUrl, callback) {
  var xhr = new XMLHttpRequest();
  xhr.open('GET', downloadUrl);
  if ( xhr.overrideMimeType ) xhr.overrideMimeType("text/plain; charset=x-user-defined");
  xhr.onload = function() {
    callback(xhr.responseText);
  };
  xhr.onerror = function() {
    callback(null);
  };
  xhr.send();
}

function ajaxJSONGet(url, callback){
  var http_request = new XMLHttpRequest();
  http_request.open("GET", url, true);
  http_request.onreadystatechange = function () {
    var done = 4;
    var ok = 200;
    if (http_request.readyState === done && http_request.status === ok){
      callback(JSON.parse(http_request.responseText));
    }
  };
  http_request.send();
}

function waitFor(handler, next, timeout) {
  timeout = timeout || 100;
  var result = handler ? handler(): true;

  if (! result) {
    setTimeout(function() {
      waitFor(handler, next, timeout);
    }, timeout);
  } else {
    next && next();
  }
}

function base64Encode(str) {
  var CHARS = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
  var out = "", i = 0, len = str.length, c1, c2, c3;
  while (i < len) {
    c1 = str.charCodeAt(i++) & 0xff;
    if (i == len) {
      out += CHARS.charAt(c1 >> 2);
      out += CHARS.charAt((c1 & 0x3) << 4);
      out += "==";
      break;
    }
    c2 = str.charCodeAt(i++);
    if (i == len) {
      out += CHARS.charAt(c1 >> 2);
      out += CHARS.charAt(((c1 & 0x3)<< 4) | ((c2 & 0xF0) >> 4));
      out += CHARS.charAt((c2 & 0xF) << 2);
      out += "=";
      break;
    }
    c3 = str.charCodeAt(i++);
    out += CHARS.charAt(c1 >> 2);
    out += CHARS.charAt(((c1 & 0x3) << 4) | ((c2 & 0xF0) >> 4));
    out += CHARS.charAt(((c2 & 0xF) << 2) | ((c3 & 0xC0) >> 6));
    out += CHARS.charAt(c3 & 0x3F);
  }
  return out;
}

function openPickDialog() {
  if(username){
    initDialog();
    getFlickrPhotos();
    activate('dialog');
  }else {
    window.open("/flickr/login", '', 'width=800,height=500');
  }
}

function findParentNode(node, attrName) {
  if (! node) {
    return null;
  }

  while (node) {
    if (node.getAttribute && node.getAttribute(attrName) !== null) {
      break;
    }
    node = node.parentNode;
  }

  return node;
}
function initDialog() {
  var html = dialogTemplate.render();

  var bodyEl = document.getElementsByTagName('BODY')[0];
  var d = document.getElementById("photopicker");
  if(d){
    bodyEl.removeChild(d);
  }

  bodyEl.insertAdjacentHTML('beforeend', html);

  var waitingHtml = loadingTemplate.render();
  var w = document.getElementById("waiting");
  if(w){
    bodyEl.removeChild(w);
  }
  bodyEl.insertAdjacentHTML('beforeend', waitingHtml);

  var cancelBtn = document.getElementById('photopicker-cancel-btn');
  cancelBtn.addEventListener('click', function(event) {
    event.preventDefault();
    event.stopPropagation();
    var listEl = document.getElementById('photopicker-albums-list');
    listEl.style.width = "300px";

    var listEl = document.getElementById('photopicker-album-files');
    listEl.style.width = "420px";

    deactivate('dialog');

    //callback && callback(null);
  }, false);

  var listEl = document.getElementById('photopicker-albums-list');
  listEl.style.width = "0px";

  var listEl = document.getElementById('photopicker-album-files');
  listEl.removeEventListener('click', photoOnSelected);
  listEl.addEventListener('click', photoOnSelected);
  listEl.style.width = "966px";
  document.getElementById('photopicker-album-files').addEventListener("scroll", function (event) {
    checkForNewDiv();
  });

  var listEls = document.getElementById('scroll-content');
  while (listEls.firstChild) {
    listEls.removeChild(listEls.firstChild);
  }

  var checkForNewDiv = function() {
    if(loading == true)
      return;
    var wrapper = document.getElementById('photopicker-album-files');
    var lastDiv = document.querySelector("#scroll-content > div:last-child");
    var lastDivOffset = lastDiv.offsetTop + lastDiv.clientHeight;
    var pageOffset = wrapper.scrollTop + wrapper.clientHeight;

    if (pageOffset > lastDivOffset - 240) {
      loading = true;
      getFlickrPhotos(currentId, currentCb, false)
    }
  };
}

function photoOnSelected(event) {
  var target = event.target;
  var result = [];

  if (target !== null && target.getAttribute('data-src')) {
    waitingActivate('dialog');
    event.preventDefault();
    event.stopPropagation();

    var source = target.getAttribute('data-src');
    page = 1;
    console.log("photo selected");
    console.log(source);

    var listEl = document.getElementById('photopicker-albums-list');
    listEl.style.width = "300px";

    var listEl = document.getElementById('photopicker-album-files');
    listEl.style.width = "420px";

    deactivate('dialog');

    downloadFile(source, (function(file) {
      var ext = file.toLowerCase().match(/\.(jpg|jpeg|png|gif)/);
      ext = typeof ext[1] !== 'undefined' ? ext[1]: null;
      var mimeType = null;

      switch (ext) {
        case 'png': mimeType = 'image/png'; break;
        case 'gif': mimeType = 'image/gif'; break;
        case 'jpg':
        case 'jpeg':
        default: mimeType = 'image/jpeg'; break;
      }

      return function(content) {
        setTimeout(function() {
          result.push('data:' + mimeType + ';base64,' + base64Encode(content));
        }, 1);
      };
    })(source));

    waitFor(function() {
      return 1 === result.length;
    }, function() {
      waitingDeactivate('dialog');
      parentCallback && parentCallback(result);
    });
  } else {
    waitingDeactivate('dialog');
    parentCallback && parentCallback(null);
  }
}

var page = 1;
var currentId = null;
var currentCb = null;
var loading = false;

function getFlickrPhotos(id, cb, removeOld) {
  currentId = id;
  currentCb = cb;

  activate('dialog-loading');

  var url = "/flickr/getPhotos";
  if(page){
    url += "?page=" + page;
  }
  console.log("getPhotos is called " + url);
  ajaxJSONGet(url, function(e){
    loading = false;
    deactivate('dialog-loading');
    var listEl = document.getElementById('scroll-content');
    if(removeOld != false) {
      while (listEl.firstChild) {
        listEl.removeChild(listEl.firstChild);
      }
    }
    if(e && e.photos && e.photos.photo) {
      for (var i = 0; i < e.photos.photo.length; i++) {
        var base = "https://farm" + e.photos.photo[i].farm + ".staticflickr.com/" + e.photos.photo[i].server + "/" + e.photos.photo[i].id + "_" + e.photos.photo[i].secret;
        var p = {id: e.photos.photo[i].id, source: base + ".jpg", picture: base + "_m.jpg"};
        listEl.insertAdjacentHTML('beforeend', photoItemTemplate.render(p));
      }
    }
    listEl.style.webkitTransform = 'scale(1)';
    if(e && e.photos && e.photos.photo && e.photos.photo.length > 0) {
      page += 1;
    }else{
      loading = true;
    }
    cb && cb();
  });
}

function activate(id) {
  var el = document.getElementById('photopicker-' + id);

  if (el) {
    el.classList.add('active');
  }
}

function deactivate(id) {
  var el = document.getElementById('photopicker-' + id);

  if (el) {
    el.classList.remove('active');
  }
}

// Exports

module.exports = function(options, callback) {

  if (typeof callback !== undefined) {
    parentCallback = callback;
  }

  if (typeof options.multiselect !== 'undefined') {
    multiselect = !! options.multiselect;
  }

  if (typeof options.clientId !== 'undefined') {
    clientId = options.clientId;
  }

  if(typeof options.username !== 'undefined'){
    username = options.username;
  }


  if (! inited) {
    inited = true;
  }

  openPickDialog();
};
}, {"./templates/file-picker.hogan":27,"./templates/album-item.hogan":28,"./templates/photo-item.hogan":9,"./templates/loading.hogan":26}],
17: [function(require, module, exports) {
"use strict";


// Varibles

var clientId;
var redirectUri;
var inited = false;
var parentCallback = null;
var multiselect = false;
var dialogTemplate = require('./templates/file-picker.hogan');
var sourceItemTemplate = require('./templates/album-item.hogan');
var photoItemTemplate = require('./templates/photo-item.hogan');
var token;
var username;
var loadingTemplate = require('./templates/loading.hogan');
// Functions
function waitingActivate(id) {
    var el = document.getElementById('waiting-' + id);

    if (el) {
        el.classList.add('active');
    }
}

function waitingDeactivate(id) {
    var el = document.getElementById('waiting-' + id);

    if (el) {
        el.classList.remove('active');
    }
}
function downloadFile(downloadUrl, callback) {
    var xhr = new XMLHttpRequest();
    xhr.open('GET', downloadUrl);
    if ( xhr.overrideMimeType ) xhr.overrideMimeType("text/plain; charset=x-user-defined");
    xhr.onload = function() {
        callback(xhr.responseText);
    };
    xhr.onerror = function() {
        callback(null);
    };
    xhr.send();
}

function ajaxJSONGet(url, callback){
    var http_request = new XMLHttpRequest();
    http_request.open("GET", url, true);
    http_request.onreadystatechange = function () {
        var done = 4;
        var ok = 200;
        if (http_request.readyState === done && http_request.status === ok){
            callback(JSON.parse(http_request.responseText));
        }
    };
    http_request.send();
}

function waitFor(handler, next, timeout) {
    timeout = timeout || 100;
    var result = handler ? handler(): true;

    if (! result) {
        setTimeout(function() {
            waitFor(handler, next, timeout);
        }, timeout);
    } else {
        next && next();
    }
}

function base64Encode(str) {
    var CHARS = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
    var out = "", i = 0, len = str.length, c1, c2, c3;
    while (i < len) {
        c1 = str.charCodeAt(i++) & 0xff;
        if (i == len) {
            out += CHARS.charAt(c1 >> 2);
            out += CHARS.charAt((c1 & 0x3) << 4);
            out += "==";
            break;
        }
        c2 = str.charCodeAt(i++);
        if (i == len) {
            out += CHARS.charAt(c1 >> 2);
            out += CHARS.charAt(((c1 & 0x3)<< 4) | ((c2 & 0xF0) >> 4));
            out += CHARS.charAt((c2 & 0xF) << 2);
            out += "=";
            break;
        }
        c3 = str.charCodeAt(i++);
        out += CHARS.charAt(c1 >> 2);
        out += CHARS.charAt(((c1 & 0x3) << 4) | ((c2 & 0xF0) >> 4));
        out += CHARS.charAt(((c2 & 0xF) << 2) | ((c3 & 0xC0) >> 6));
        out += CHARS.charAt(c3 & 0x3F);
    }
    return out;
}

function openPickDialog() {
    if(token){
        getAlbums();
    }else {
        window.open("/instagram/login?filepicker=true", '', 'width=800,height=500');
    }
}

function getAlbums(){
    initDialog();
    console.log("get instagram albums");

    var listEl = document.getElementById('photopicker-albums-list');
    while (listEl.firstChild) {
        listEl.removeChild(listEl.firstChild);
    }
    listEl.insertAdjacentHTML('beforeend', sourceItemTemplate.render({id: 'self', name: "My photos"}));
    listEl.insertAdjacentHTML('beforeend', sourceItemTemplate.render({id: 'friends', name: "Feed"}));
    getInstagramPhotos('self', function() {
        deactivate('dialog-loading');
    });
    ajaxJSONGet("/instagram/getFollowing", function(e) {
        for(var i = 0; i < e.length;i++){
            listEl.insertAdjacentHTML('beforeend', sourceItemTemplate.render({id: e[i].id, name: e[i].username}));
        }
    });
    activate('dialog');
}
function findParentNode(node, attrName) {
    if (! node) {
        return null;
    }

    while (node) {
        if (node.getAttribute && node.getAttribute(attrName) !== null) {
            break;
        }
        node = node.parentNode;
    }

    return node;
}
function initDialog() {
    var html = dialogTemplate.render();

    var bodyEl = document.getElementsByTagName('BODY')[0];
    var d = document.getElementById("photopicker");
    if(d){
        bodyEl.removeChild(d);
    }

    bodyEl.insertAdjacentHTML('beforeend', html);

    var waitingHtml = loadingTemplate.render();
    var w = document.getElementById("waiting");
    if(w){
        bodyEl.removeChild(w);
    }
    bodyEl.insertAdjacentHTML('beforeend', waitingHtml);

    var cancelBtn = document.getElementById('photopicker-cancel-btn');
    cancelBtn.addEventListener('click', function(event) {
        event.preventDefault();
        event.stopPropagation();

        var listEl = document.getElementById('photopicker-albums-list');
        listEl.removeEventListener('click', albumOnSelected);

        var listEl = document.getElementById('photopicker-album-files');
        listEl.removeEventListener('click', photoOnSelected);

        deactivate('dialog');

        //callback && callback(null);
    }, false);

    var listEl = document.getElementById('photopicker-albums-list');
    listEl.removeEventListener('click', albumOnSelected);
    listEl.addEventListener('click', albumOnSelected);

    var listEl = document.getElementById('photopicker-album-files');
    listEl.removeEventListener('click', photoOnSelected);
    listEl.addEventListener('click', photoOnSelected);
    document.getElementById('photopicker-album-files').addEventListener("scroll", function (event) {
        checkForNewDiv();
    });

    var listEls = document.getElementById('scroll-content');
    while (listEls.firstChild) {
        listEls.removeChild(listEls.firstChild);
    }

    var checkForNewDiv = function() {
        if(loading == true) return;
        var wrapper = document.getElementById('photopicker-album-files');
        var lastDiv = document.querySelector("#scroll-content > div:last-child");
        var lastDivOffset = lastDiv.offsetTop + lastDiv.clientHeight;
        var pageOffset = wrapper.scrollTop + wrapper.clientHeight;

        if (pageOffset > lastDivOffset - 240) {
            loading = true;
            getInstagramPhotos(currentId, currentCb, false)
        }
    };
}
function albumOnSelected(event) {
    var target = findParentNode(event.target, 'data-id');

    if (target !== null) {
        event.preventDefault();
        event.stopPropagation();

        var sourceId = target.getAttribute('data-id');
        activate('dialog-loading');
        afterCursor = null;
        getInstagramPhotos(sourceId, function() {
            deactivate('dialog-loading');
        });
    }
}

function photoOnSelected(event) {
    var target = event.target;
    var result = [];

    if (target !== null && target.getAttribute('data-src')) {
        waitingActivate('dialog');
        event.preventDefault();
        event.stopPropagation();

        var listEl = document.getElementById('photopicker-albums-list');
        listEl.removeEventListener('click', albumOnSelected);

        var listEl = document.getElementById('photopicker-album-files');
        listEl.removeEventListener('click', photoOnSelected);

        var source = target.getAttribute('data-src');
        afterCursor = null;
        console.log("photo selected");
        console.log(source);
        deactivate('dialog');

        downloadFile(source, (function(file) {
            var ext = file.toLowerCase().match(/\.(jpg|jpeg|png|gif)/);
            ext = typeof ext[1] !== 'undefined' ? ext[1]: null;
            var mimeType = null;

            switch (ext) {
                case 'png': mimeType = 'image/png'; break;
                case 'gif': mimeType = 'image/gif'; break;
                case 'jpg':
                case 'jpeg':
                default: mimeType = 'image/jpeg'; break;
            }

            return function(content) {
                setTimeout(function() {
                    result.push('data:' + mimeType + ';base64,' + base64Encode(content));
                }, 1);
            };
        })(source));

        waitFor(function() {
            return 1 === result.length;
        }, function() {
            waitingDeactivate('dialog');
            parentCallback && parentCallback(result);
        });
    } else {
        waitingDeactivate('dialog');
        parentCallback && parentCallback(null);
    }
}

var afterCursor = null;
var currentId = null;
var currentCb = null;
var loading = false;

function getInstagramPhotos(id, cb, removeOld) {
    currentId = id;
    currentCb = cb;
    var url = "/instagram/getFeed";
    if(id == "self"){
        url = "/instagram/getUserMedia";
        if(afterCursor){
            url += "?max_id=" + afterCursor;
        }
    }else if(id == "friends"){
        url = "/instagram/getFeed";
        if(afterCursor){
            url += "?max_id=" + afterCursor;
        }
    }else{
        url = "/instagram/getUsersMedia?user_id=" + id;
        if(afterCursor){
            url += "&max_id=" + afterCursor;
        }
    }
    console.log("getPhotos is called " + url);
    activate('dialog-loading');
    ajaxJSONGet(url, function(e){
        deactivate('dialog-loading');
        loading = false;
        var listEl = document.getElementById('scroll-content');
        if(removeOld != false) {
            while (listEl.firstChild) {
                listEl.removeChild(listEl.firstChild);
            }
        }
        for(var i = 0; i < e.length;i++){
            var p = {id: e[i].id, source: e[i].image_large, picture: e[i].image_thumbnail};
            listEl.insertAdjacentHTML('beforeend', photoItemTemplate.render(p));
            afterCursor = p.id;
        }
        listEl.style.webkitTransform = 'scale(1)';
        if(e && e.length > 0) {
        }else{
            loading = true;
        }
        cb && cb();
    });
}

function activate(id) {
    var el = document.getElementById('photopicker-' + id);

    if (el) {
        el.classList.add('active');
    }
}

function deactivate(id) {
    var el = document.getElementById('photopicker-' + id);

    if (el) {
        el.classList.remove('active');
    }
}

// Exports

module.exports = function(options, callback) {

    if (typeof callback !== undefined) {
        parentCallback = callback;
    }

    if (typeof options.multiselect !== 'undefined') {
        multiselect = !! options.multiselect;
    }

    if (typeof options.clientId !== 'undefined') {
        clientId = options.clientId;
    }

    if(typeof options.username !== 'undefined'){
        username = options.username;
    }

    if(typeof options.token !== 'undefined'){
        token = options.token;
    }


    if (! inited) {
        inited = true;
    }

    openPickDialog();
};
}, {"./templates/file-picker.hogan":27,"./templates/album-item.hogan":28,"./templates/photo-item.hogan":9,"./templates/loading.hogan":26}],
18: [function(require, module, exports) {
"use strict";


// Varibles

var clientId;
var redirectUri;
var inited = false;
var parentCallback = null;
var multiselect = false;
var dialogTemplate = require('./templates/file-picker.hogan');
var sourceItemTemplate = require('./templates/album-item.hogan');
var photoItemTemplate = require('./templates/photo-item.hogan');
var token;
var username;
var loadingTemplate = require('./templates/loading.hogan');
// Functions
function waitingActivate(id) {
    var el = document.getElementById('waiting-' + id);

    if (el) {
        el.classList.add('active');
    }
}

function waitingDeactivate(id) {
    var el = document.getElementById('waiting-' + id);

    if (el) {
        el.classList.remove('active');
    }
}
function downloadFile(downloadUrl, callback) {
    var xhr = new XMLHttpRequest();
    xhr.open('GET', downloadUrl);
    if ( xhr.overrideMimeType ) xhr.overrideMimeType("text/plain; charset=x-user-defined");
    xhr.onload = function() {
        callback(xhr.responseText);
    };
    xhr.onerror = function() {
        callback(null);
    };
    xhr.send();
}

function ajaxJSONGet(url, callback){
    var http_request = new XMLHttpRequest();
    http_request.open("GET", url, true);
    http_request.onreadystatechange = function () {
        var done = 4;
        var ok = 200;
        if (http_request.readyState === done && http_request.status === ok){
            callback(JSON.parse(http_request.responseText));
        }
    };
    http_request.send();
}

function waitFor(handler, next, timeout) {
    timeout = timeout || 100;
    var result = handler ? handler(): true;

    if (! result) {
        setTimeout(function() {
            waitFor(handler, next, timeout);
        }, timeout);
    } else {
        next && next();
    }
}

function base64Encode(str) {
    var CHARS = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
    var out = "", i = 0, len = str.length, c1, c2, c3;
    while (i < len) {
        c1 = str.charCodeAt(i++) & 0xff;
        if (i == len) {
            out += CHARS.charAt(c1 >> 2);
            out += CHARS.charAt((c1 & 0x3) << 4);
            out += "==";
            break;
        }
        c2 = str.charCodeAt(i++);
        if (i == len) {
            out += CHARS.charAt(c1 >> 2);
            out += CHARS.charAt(((c1 & 0x3)<< 4) | ((c2 & 0xF0) >> 4));
            out += CHARS.charAt((c2 & 0xF) << 2);
            out += "=";
            break;
        }
        c3 = str.charCodeAt(i++);
        out += CHARS.charAt(c1 >> 2);
        out += CHARS.charAt(((c1 & 0x3) << 4) | ((c2 & 0xF0) >> 4));
        out += CHARS.charAt(((c2 & 0xF) << 2) | ((c3 & 0xC0) >> 6));
        out += CHARS.charAt(c3 & 0x3F);
    }
    return out;
}

function openPickDialog() {
    if(token){
        getAlbums();
    }else {
        window.open("/picasa/login?filepicker=true", '', 'width=800,height=500');
    }
}

function getAlbums(){
    initDialog();
    console.log("get picasa albums");

    var listEl = document.getElementById('photopicker-albums-list');
    while (listEl.firstChild) {
        listEl.removeChild(listEl.firstChild);
    }
    ajaxJSONGet("/picasa/albums", function(e) {
        for(var i = 0; i < e.length;i++){
            if(i == 0){
                getPicasaPhotos(e[i].id, function() {
                    deactivate('dialog-loading');
                });
            }
            listEl.insertAdjacentHTML('beforeend', sourceItemTemplate.render({id: e[i].id, name: e[i].title}));
        }
    });
    activate('dialog');
}
function findParentNode(node, attrName) {
    if (! node) {
        return null;
    }

    while (node) {
        if (node.getAttribute && node.getAttribute(attrName) !== null) {
            break;
        }
        node = node.parentNode;
    }

    return node;
}
function initDialog() {
    var html = dialogTemplate.render();

    var bodyEl = document.getElementsByTagName('BODY')[0];
    var d = document.getElementById("photopicker");
    if(d){
        bodyEl.removeChild(d);
    }

    bodyEl.insertAdjacentHTML('beforeend', html);

    var waitingHtml = loadingTemplate.render();
    var w = document.getElementById("waiting");
    if(w){
        bodyEl.removeChild(w);
    }
    bodyEl.insertAdjacentHTML('beforeend', waitingHtml);

    var cancelBtn = document.getElementById('photopicker-cancel-btn');
    cancelBtn.addEventListener('click', function(event) {
        event.preventDefault();
        event.stopPropagation();
        var listEl = document.getElementById('photopicker-albums-list');
        listEl.removeEventListener('click', albumOnSelected);

        var listEl = document.getElementById('photopicker-album-files');
        listEl.removeEventListener('click', photoOnSelected);

        deactivate('dialog');

        //callback && callback(null);
    }, false);

    var listEl = document.getElementById('photopicker-albums-list');
    listEl.removeEventListener('click', albumOnSelected);
    listEl.addEventListener('click', albumOnSelected);

    var listEl = document.getElementById('photopicker-album-files');
    listEl.removeEventListener('click', photoOnSelected);
    listEl.addEventListener('click', photoOnSelected);
    document.getElementById('photopicker-album-files').addEventListener("scroll", function (event) {
        checkForNewDiv();
    });

    var listEls = document.getElementById('scroll-content');
    while (listEls.firstChild) {
        listEls.removeChild(listEls.firstChild);
    }

    var checkForNewDiv = function() {
        if(loading == true) return;
        var wrapper = document.getElementById('photopicker-album-files');
        var lastDiv = document.querySelector("#scroll-content > div:last-child");
        var lastDivOffset = lastDiv.offsetTop + lastDiv.clientHeight;
        var pageOffset = wrapper.scrollTop + wrapper.clientHeight;

        if (pageOffset > lastDivOffset - 240) {
            loading = true;
            getPicasaPhotos(currentId, currentCb, false)
        }
    };
}
function albumOnSelected(event) {
    var target = findParentNode(event.target, 'data-id');

    if (target !== null) {
        event.preventDefault();
        event.stopPropagation();

        var sourceId = target.getAttribute('data-id');
        activate('dialog-loading');
        afterCursor = 0;
        getPicasaPhotos(sourceId, function() {
            deactivate('dialog-loading');
        });
    }
}

function photoOnSelected(event) {
    var target = event.target;
    var result = [];

    if (target !== null && target.getAttribute('data-src')) {
        waitingActivate('dialog');
        event.preventDefault();
        event.stopPropagation();

        var listEl = document.getElementById('photopicker-albums-list');
        listEl.removeEventListener('click', albumOnSelected);

        var listEl = document.getElementById('photopicker-album-files');
        listEl.removeEventListener('click', photoOnSelected);
        var source = target.getAttribute('data-src');
        afterCursor = 0;
        console.log("photo selected");
        console.log(source);
        deactivate('dialog');

        downloadFile(source, (function(file) {
            var ext = file.toLowerCase().match(/\.(jpg|jpeg|png|gif)/);
            ext = typeof ext[1] !== 'undefined' ? ext[1]: null;
            var mimeType = null;

            switch (ext) {
                case 'png': mimeType = 'image/png'; break;
                case 'gif': mimeType = 'image/gif'; break;
                case 'jpg':
                case 'jpeg':
                default: mimeType = 'image/jpeg'; break;
            }

            return function(content) {
                setTimeout(function() {
                    result.push('data:' + mimeType + ';base64,' + base64Encode(content));
                }, 1);
            };
        })(source));

        waitFor(function() {
            return 1 === result.length;
        }, function() {
            waitingDeactivate('dialog');
            parentCallback && parentCallback(result);
        });
    } else {
        waitingDeactivate('dialog');
        parentCallback && parentCallback(null);
    }
}

var afterCursor = 0;
var currentId = null;
var currentCb = null;
var loading = false;
var totalCount = 0;

function getPicasaPhotos(id, cb, removeOld) {
    currentId = id;
    currentCb = cb;
    var url = "/picasa/albums?albumName=&albumId=" + id;
    if(afterCursor > 0){
        url += "&start=" + (afterCursor + 1);
    }
    activate('dialog-loading');
    ajaxJSONGet(url, function(e){
        deactivate('dialog-loading');
        loading = false;
        var listEl = document.getElementById('scroll-content');
        if(removeOld != false) {
            while (listEl.firstChild) {
                listEl.removeChild(listEl.firstChild);
            }
        }
        for(var i = 0; i < e.photos.length;i++){
            var p = {id: e.photos[i].id, source: e.photos[i].url, picture: e.photos[i].thumbnail};
            listEl.insertAdjacentHTML('beforeend', photoItemTemplate.render(p));
            afterCursor++;
        }
        listEl.style.webkitTransform = 'scale(1)';
        if(e && e.photos && e.photos.length > 0) {
        }else{
            loading = true;
        }
        totalCount = e.count;
        cb && cb();
    });
}

function activate(id) {
    var el = document.getElementById('photopicker-' + id);

    if (el) {
        el.classList.add('active');
    }
}

function deactivate(id) {
    var el = document.getElementById('photopicker-' + id);

    if (el) {
        el.classList.remove('active');
    }
}

// Exports

module.exports = function(options, callback) {

    if (typeof callback !== undefined) {
        parentCallback = callback;
    }

    if (typeof options.multiselect !== 'undefined') {
        multiselect = !! options.multiselect;
    }

    if (typeof options.clientId !== 'undefined') {
        clientId = options.clientId;
    }

    if(typeof options.username !== 'undefined'){
        username = options.username;
    }

    if(typeof options.token !== 'undefined'){
        token = options.token;
    }


    if (! inited) {
        inited = true;
    }

    openPickDialog();
};
}, {"./templates/file-picker.hogan":27,"./templates/album-item.hogan":28,"./templates/photo-item.hogan":9,"./templates/loading.hogan":26}],
19: [function(require, module, exports) {
"use strict";


// Varibles

var clientId;
var redirectUri;
var inited = false;
var parentCallback = null;
var multiselect = false;
var dialogTemplate = require('./templates/file-picker.hogan');
var sourceItemTemplate = require('./templates/asset-folder.hogan');
var photoItemTemplate = require('./templates/photo-item.hogan');
var loadingTemplate = require('./templates/loading.hogan');
var token;
var username;
var assets;
// Functions

function waitingActivate(id) {
    var el = document.getElementById('waiting-' + id);

    if (el) {
        el.classList.add('active');
    }
}

function waitingDeactivate(id) {
    var el = document.getElementById('waiting-' + id);

    if (el) {
        el.classList.remove('active');
    }
}
function downloadFile(downloadUrl, callback) {
    var xhr = new XMLHttpRequest();
    xhr.open('GET', downloadUrl);
    if ( xhr.overrideMimeType ) xhr.overrideMimeType("text/plain; charset=x-user-defined");
    xhr.onload = function() {
        callback(xhr.responseText);
    };
    xhr.onerror = function() {
        callback(null);
    };
    xhr.send();
}

function ajaxJSONGet(url, callback){
    var http_request = new XMLHttpRequest();
    http_request.open("GET", url, true);
    http_request.onreadystatechange = function () {
        var done = 4;
        var ok = 200;
        if (http_request.readyState === done && http_request.status === ok){
            callback(JSON.parse(http_request.responseText));
        }
    };
    http_request.send();
}

function waitFor(handler, next, timeout) {
    timeout = timeout || 100;
    var result = handler ? handler(): true;

    if (! result) {
        setTimeout(function() {
            waitFor(handler, next, timeout);
        }, timeout);
    } else {
        next && next();
    }
}

function base64Encode(str) {
    var CHARS = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
    var out = "", i = 0, len = str.length, c1, c2, c3;
    while (i < len) {
        c1 = str.charCodeAt(i++) & 0xff;
        if (i == len) {
            out += CHARS.charAt(c1 >> 2);
            out += CHARS.charAt((c1 & 0x3) << 4);
            out += "==";
            break;
        }
        c2 = str.charCodeAt(i++);
        if (i == len) {
            out += CHARS.charAt(c1 >> 2);
            out += CHARS.charAt(((c1 & 0x3)<< 4) | ((c2 & 0xF0) >> 4));
            out += CHARS.charAt((c2 & 0xF) << 2);
            out += "=";
            break;
        }
        c3 = str.charCodeAt(i++);
        out += CHARS.charAt(c1 >> 2);
        out += CHARS.charAt(((c1 & 0x3) << 4) | ((c2 & 0xF0) >> 4));
        out += CHARS.charAt(((c2 & 0xF) << 2) | ((c3 & 0xC0) >> 6));
        out += CHARS.charAt(c3 & 0x3F);
    }
    return out;
}

function openPickDialog() {
    getAlbums();
}

function getAlbums(){
    console.log("get pizap background albums");
    initDialog();

    var listEl = document.getElementById('photopicker-albums-list');
    while (listEl.firstChild) {
        listEl.removeChild(listEl.firstChild);
    }
    var listEls = document.getElementById('scroll-content');
    while (listEls.firstChild) {
        listEls.removeChild(listEls.firstChild);
    }
    waitingActivate('dialog');

    downloadFile("/assets.json", function(content) {
        assets = JSON.parse(content);
        var listEl = document.getElementById('photopicker-albums-list');
        while (listEl.firstChild) {
            listEl.removeChild(listEl.firstChild);
        }
        var basePath = assets.CDNPath + assets.Backgrounds.Path;
        if('https:' == document.location.protocol){
            basePath = assets.CDNSSLPath + assets.Backgrounds.Path;
        }
        var last = "";

        for(var i = 0; i < assets.Backgrounds.items.length;i++) {
            if(last != assets.Backgrounds.items[i].Name) {
                if(i == 0){
                    getBackgroundPhotos(assets.Backgrounds.items[i].Name, function() {
                        deactivate('dialog-loading');
                    });

                }
                listEl.insertAdjacentHTML('beforeend', sourceItemTemplate.render({
                    id: assets.Backgrounds.items[i].Name,
                    name: assets.Backgrounds.items[i].Name,
                    icon: basePath + assets.Backgrounds.items[i].Thumb
                }));
            }
            last = assets.Backgrounds.items[i].Name;
        }
        waitingDeactivate('dialog');

    });

    activate('dialog');
}
function findParentNode(node, attrName) {
    if (! node) {
        return null;
    }

    while (node) {
        if (node.getAttribute && node.getAttribute(attrName) !== null) {
            break;
        }
        node = node.parentNode;
    }

    return node;
}
function initDialog() {
    var html = dialogTemplate.render();

    var bodyEl = document.getElementsByTagName('BODY')[0];
    var d = document.getElementById("photopicker");
    if(d){
        bodyEl.removeChild(d);
    }
    bodyEl.insertAdjacentHTML('beforeend', html);

    var waitingHtml = loadingTemplate.render();
    var w = document.getElementById("waiting");
    if(w){
        bodyEl.removeChild(w);
    }
    bodyEl.insertAdjacentHTML('beforeend', waitingHtml);

    var cancelBtn = document.getElementById('photopicker-cancel-btn');
    cancelBtn.addEventListener('click', function(event) {
        event.preventDefault();
        event.stopPropagation();

        var listEl = document.getElementById('photopicker-albums-list');
        listEl.removeEventListener('click', albumOnSelected);

        var listEl = document.getElementById('photopicker-album-files');
        listEl.removeEventListener('click', photoOnSelected);

        deactivate('dialog');

        //callback && callback(null);
    }, false);

    var listEl = document.getElementById('photopicker-albums-list');
    listEl.removeEventListener('click', albumOnSelected);
    listEl.addEventListener('click', albumOnSelected);

    var listEl = document.getElementById('photopicker-album-files');
    listEl.removeEventListener('click', photoOnSelected);
    listEl.addEventListener('click', photoOnSelected);
}
function albumOnSelected(event) {
    var target = findParentNode(event.target, 'data-id');

    if (target !== null) {
        event.preventDefault();
        event.stopPropagation();

        var sourceId = target.getAttribute('data-id');
        activate('dialog-loading');
        getBackgroundPhotos(sourceId, function() {
            deactivate('dialog-loading');
        });
    }
}

function photoOnSelected(event) {
    var target = event.target;
    var result = [];

    waitingActivate('dialog');
    if (target !== null && target.getAttribute('data-src')) {
        event.preventDefault();
        event.stopPropagation();
        var listEl = document.getElementById('photopicker-albums-list');
        listEl.removeEventListener('click', albumOnSelected);

        var listEl = document.getElementById('photopicker-album-files');
        listEl.removeEventListener('click', photoOnSelected);

        var source = target.getAttribute('data-src');
        console.log("photo selected");
        console.log(source);
        deactivate('dialog');

        downloadFile(source, (function(file) {
            var ext = file.toLowerCase().match(/\.(jpg|jpeg|png|gif)/);
            ext = typeof ext[1] !== 'undefined' ? ext[1]: null;
            var mimeType = null;

            switch (ext) {
                case 'png': mimeType = 'image/png'; break;
                case 'gif': mimeType = 'image/gif'; break;
                case 'jpg':
                case 'jpeg':
                default: mimeType = 'image/jpeg'; break;
            }

            return function(content) {
                setTimeout(function() {
                    result.push('data:' + mimeType + ';base64,' + base64Encode(content));
                }, 1);
            };
        })(source));

        waitFor(function() {
            return 1 === result.length;
        }, function() {
            waitingDeactivate('dialog');
            parentCallback && parentCallback(result);
        });
    } else {
        waitingDeactivate('dialog');
        parentCallback && parentCallback(null);
    }
}
function zeroPad(num, places) {
    var zero = places - num.toString().length + 1;
    return Array(+(zero > 0 && zero)).join("0") + num;
}
function getBackgroundPhotos(id, cb, removeOld) {
    activate('dialog-loading');
    var listEl = document.getElementById('scroll-content');
    if(removeOld != false) {
        while (listEl.firstChild) {
            listEl.removeChild(listEl.firstChild);
        }
    }

    if(assets && assets.Backgrounds && assets.Backgrounds.items){
        for(var i = 0; i < assets.Backgrounds.items.length;i++) {
            if(assets.Backgrounds.items[i].Name === id){
                var item = assets.Backgrounds.items[i];
                var basePath = assets.CDNPath + assets.Backgrounds.Path + item.Path;
                if('https:' == document.location.protocol){
                    basePath = assets.CDNSSLPath + assets.Backgrounds.Path + item.Path;
                }
                var thumbnailExt = "jpg";
                var thumbnailPrefix = "thumb";
                var thumbnailDigits = 1;
                var pathPrefix = "img";
                var pathExt = "jpg";
                var pathDigits = 1;

                if(item.ThumbnailExt) thumbnailExt = item.ThumbnailExt;
                if(item.ThumbnailPrefix) thumbnailPrefix = item.ThumbnailPrefix;
                if(item.ThumbnailDigits) thumbnailDigits = item.ThumbnailDigits;

                if(item.PathExt) pathExt = item.PathExt;
                if(item.PathPrefix) pathPrefix = item.PathPrefix;
                if(item.PathDigits) pathDigits = item.PathDigits;

                for(var j = 1; j <= item.ItemCount;j++){
                    var thumb = basePath + thumbnailPrefix + zeroPad(j, thumbnailDigits) + "." + thumbnailExt;
                    var source = basePath + pathPrefix + zeroPad(j, pathDigits) + "." + pathExt;

                    var p = {id: id + j, source: source, picture: thumb};
                    listEl.insertAdjacentHTML('beforeend', photoItemTemplate.render(p));
                }
            }
        }

    }
    listEl.style.webkitTransform = 'scale(1)';
    cb && cb();
}

function activate(id) {
    var el = document.getElementById('photopicker-' + id);

    if (el) {
        el.classList.add('active');
    }
}

function deactivate(id) {
    var el = document.getElementById('photopicker-' + id);

    if (el) {
        el.classList.remove('active');
    }
}

// Exports

module.exports = function(options, callback) {

    if (typeof callback !== undefined) {
        parentCallback = callback;
    }

    if (typeof options.multiselect !== 'undefined') {
        multiselect = !! options.multiselect;
    }

    if (typeof options.clientId !== 'undefined') {
        clientId = options.clientId;
    }

    if(typeof options.username !== 'undefined'){
        username = options.username;
    }

    if(typeof options.token !== 'undefined'){
        token = options.token;
    }


    if (! inited) {
        inited = true;
    }

    openPickDialog();
};
}, {"./templates/file-picker.hogan":27,"./templates/asset-folder.hogan":8,"./templates/photo-item.hogan":9,"./templates/loading.hogan":26}],
20: [function(require, module, exports) {
"use strict";
var inited = false;
var parentCallback = null;
var multiselect = false;
var loadingTemplate = require('./templates/loading.hogan');

function openPickDialog() {
    deactivate('dialog');
    var flashMovie = getFlashMovieObject('flashContent');

    flashMovie && flashMovie.startCam();
}

function activate(id) {
    var el = document.getElementById('photopicker-' + id);

    if (el) {
        el.classList.add('active');
    }
}

function deactivate(id) {
    var el = document.getElementById('photopicker-' + id);

    if (el) {
        el.classList.remove('active');
    }
}

// Exports

module.exports = function(options, callback) {

    if (typeof callback !== undefined) {
        parentCallback = callback;
    }

    if (typeof options.multiselect !== 'undefined') {
        multiselect = !! options.multiselect;
    }

    if (! inited) {
        inited = true;
    }

    openPickDialog();
};
}, {"./templates/loading.hogan":26}],
21: [function(require, module, exports) {
"use strict";


// Varibles

var inited = false;
var parentCallback = null;
var multiselect = false;
var dialogTemplate = require('./templates/file-picker.hogan');
var photoItemTemplate = require('./templates/photo-item.hogan');
var loadingTemplate = require('./templates/loading.hogan');
// Functions
function waitingActivate(id) {
    var el = document.getElementById('waiting-' + id);

    if (el) {
        el.classList.add('active');
    }
}

function waitingDeactivate(id) {
    var el = document.getElementById('waiting-' + id);

    if (el) {
        el.classList.remove('active');
    }
}
function downloadFile(downloadUrl, callback) {
    var xhr = new XMLHttpRequest();
    xhr.open('GET', downloadUrl);
    if ( xhr.overrideMimeType ) xhr.overrideMimeType("text/plain; charset=x-user-defined");
    xhr.onload = function() {
        callback(xhr.responseText);
    };
    xhr.onerror = function() {
        callback(null);
    };
    xhr.send();
}

function ajaxJSONGet(url, callback){
    var http_request = new XMLHttpRequest();
    http_request.open("GET", url, true);
    http_request.onreadystatechange = function () {
        var done = 4;
        var ok = 200;
        if (http_request.readyState === done && http_request.status === ok){
            callback(JSON.parse(http_request.responseText));
        }
    };
    http_request.send();
}

function waitFor(handler, next, timeout) {
    timeout = timeout || 100;
    var result = handler ? handler(): true;

    if (! result) {
        setTimeout(function() {
            waitFor(handler, next, timeout);
        }, timeout);
    } else {
        next && next();
    }
}

function base64Encode(str) {
    var CHARS = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
    var out = "", i = 0, len = str.length, c1, c2, c3;
    while (i < len) {
        c1 = str.charCodeAt(i++) & 0xff;
        if (i == len) {
            out += CHARS.charAt(c1 >> 2);
            out += CHARS.charAt((c1 & 0x3) << 4);
            out += "==";
            break;
        }
        c2 = str.charCodeAt(i++);
        if (i == len) {
            out += CHARS.charAt(c1 >> 2);
            out += CHARS.charAt(((c1 & 0x3)<< 4) | ((c2 & 0xF0) >> 4));
            out += CHARS.charAt((c2 & 0xF) << 2);
            out += "=";
            break;
        }
        c3 = str.charCodeAt(i++);
        out += CHARS.charAt(c1 >> 2);
        out += CHARS.charAt(((c1 & 0x3) << 4) | ((c2 & 0xF0) >> 4));
        out += CHARS.charAt(((c2 & 0xF) << 2) | ((c3 & 0xC0) >> 6));
        out += CHARS.charAt(c3 & 0x3F);
    }
    return out;
}

function openPickDialog() {
    if(proPurchased){
        initDialog();
        activate('dialog');
    }else {
        window.showStore();
    }
}

function findParentNode(node, attrName) {
    if (! node) {
        return null;
    }

    while (node) {
        if (node.getAttribute && node.getAttribute(attrName) !== null) {
            break;
        }
        node = node.parentNode;
    }

    return node;
}
function initDialog() {
    var html = dialogTemplate.render();

    var bodyEl = document.getElementsByTagName('BODY')[0];
    var d = document.getElementById("photopicker");
    if(d){
        bodyEl.removeChild(d);
    }

    bodyEl.insertAdjacentHTML('beforeend', html);
    var waitingHtml = loadingTemplate.render();
    var w = document.getElementById("waiting");
    if(w){
        bodyEl.removeChild(w);
    }
    bodyEl.insertAdjacentHTML('beforeend', waitingHtml);

    var filesElement = document.getElementById('photopicker-album-files');
    if(filesElement){
        filesElement.style.height = "405px";
        filesElement.insertAdjacentHTML('beforebegin', '<div style="margin: 5px;" align="center"><input id="googleSearchText" type="text" placeholder="Google Image Search" value="" style="width:650px; "><button class="search-btn btn" id="googleSearchBtn" type="button" style="margin-left:5px;background-color: #ccc;"><i class="icon1 icon-google" style="background-color: #ccc; margin:5px;"></i></button></div>');
    }

    var searchBtn = document.getElementById("googleSearchBtn");
    searchBtn.addEventListener('click', function(event){
        var searchText = document.getElementById("googleSearchText");
        page = 1;
        getGooglePhotos(searchText.value, null, true);
    }, false);

    var cancelBtn = document.getElementById('photopicker-cancel-btn');
    cancelBtn.addEventListener('click', function(event) {
        event.preventDefault();
        event.stopPropagation();
        var listEl = document.getElementById('photopicker-albums-list');
        listEl.style.width = "300px";

        var listEl = document.getElementById('photopicker-album-files');
        listEl.style.width = "420px";

        deactivate('dialog');

        //callback && callback(null);
    }, false);

    var listEl = document.getElementById('photopicker-albums-list');
    listEl.style.width = "0px";

    var listEl = document.getElementById('photopicker-album-files');
    listEl.removeEventListener('click', photoOnSelected);
    listEl.addEventListener('click', photoOnSelected);
    listEl.style.width = "966px";
    document.getElementById('photopicker-album-files').addEventListener("scroll", function (event) {
        checkForNewDiv();
    });

    var listEls = document.getElementById('scroll-content');
    while (listEls.firstChild) {
        listEls.removeChild(listEls.firstChild);
    }
    var checkForNewDiv = function() {
        if(loading == true)
            return;
        var wrapper = document.getElementById('photopicker-album-files');
        var lastDiv = document.querySelector("#scroll-content > div:last-child");
        var lastDivOffset = lastDiv.offsetTop + lastDiv.clientHeight;
        var pageOffset = wrapper.scrollTop + wrapper.clientHeight;

        if (pageOffset > lastDivOffset - 240) {
            loading = true;
            getGooglePhotos(currentId, currentCb, false)
        }
    };
}

function photoOnSelected(event) {
    var target = event.target;
    var result = [];

    if (target !== null && target.getAttribute('data-src')) {
        waitingActivate('dialog');
        event.preventDefault();
        event.stopPropagation();

        var source = target.getAttribute('data-src');
        source = '/google/image?url=' + encodeURI(source);
        page = 1;
        console.log("photo selected");
        console.log(source);

        var listEl = document.getElementById('photopicker-albums-list');
        listEl.style.width = "300px";

        var listEl = document.getElementById('photopicker-album-files');
        listEl.style.width = "420px";

        deactivate('dialog');

        downloadFile(source, (function(file) {
            var ext = file.toLowerCase().match(/\.(jpg|jpeg|png|gif)/);

            ext = ext && ext.length >= 2 && typeof ext[1] !== 'undefined' ? ext[1]: null;
            var mimeType = null;

            switch (ext) {
                case 'png': mimeType = 'image/png'; break;
                case 'gif': mimeType = 'image/gif'; break;
                case 'jpg':
                case 'jpeg':
                default: mimeType = 'image/jpeg'; break;
            }

            return function(content) {
                setTimeout(function() {
                    result.push('data:' + mimeType + ';base64,' + base64Encode(content));
                }, 1);
            };
        })(source));

        waitFor(function() {
            return 1 === result.length;
        }, function() {
            waitingDeactivate('dialog');
            parentCallback && parentCallback(result);
        });
    } else {
        waitingDeactivate('dialog');
        parentCallback && parentCallback(null);
    }
}

var page = 1;
var currentId = null;
var currentCb = null;
var loading = false;

function getGooglePhotos(id, cb, removeOld) {
    currentId = id;
    currentCb = cb;

    activate('dialog-loading');

    var url = "/google/search?term=" + id + "&start=" + page;
    console.log("getPhotos is called " + url);
    ajaxJSONGet(url, function(e){
        loading = false;
        deactivate('dialog-loading');
        var listEl = document.getElementById('scroll-content');
        if(removeOld != false) {
            while (listEl.firstChild) {
                listEl.removeChild(listEl.firstChild);
            }
        }
        if(e && e.items) {
            for (var i = 0; i < e.items.length; i++) {
                var p = {id: e.items[i].link, source: e.items[i].link, picture: e.items[i].image.thumbnailLink};
                listEl.insertAdjacentHTML('beforeend', photoItemTemplate.render(p));
            }
            if(e.queries && e.queries.nextPage && e.queries.nextPage[0].startIndex){
                page = e.queries.nextPage[0].startIndex;
                if(page == 11 || page == 21 || page == 31){
                    getGooglePhotos(currentId, currentCb, false)
                }
                if(page >= 100){
                    loading = true;
                }else {
                    loading = false;
                }
            }else{
                loading = true;
            }
        }
        listEl.style.webkitTransform = 'scale(1)';
        cb && cb();
    });
}

function activate(id) {
    var el = document.getElementById('photopicker-' + id);

    if (el) {
        el.classList.add('active');
    }
}

function deactivate(id) {
    var el = document.getElementById('photopicker-' + id);

    if (el) {
        el.classList.remove('active');
    }
}

// Exports

module.exports = function(options, callback) {

    if (typeof callback !== undefined) {
        parentCallback = callback;
    }

    if (typeof options.multiselect !== 'undefined') {
        multiselect = !! options.multiselect;
    }

    if (! inited) {
        inited = true;
    }

    openPickDialog();
};
}, {"./templates/file-picker.hogan":27,"./templates/photo-item.hogan":9,"./templates/loading.hogan":26}],
22: [function(require, module, exports) {
"use strict";


// Varibles

var clientId;
var redirectUri;
var inited = false;
var parentCallback = null;
var multiselect = false;
var dialogTemplate = require('./templates/file-picker.hogan');
var sourceItemTemplate = require('./templates/asset-folder.hogan');
var photoItemTemplate = require('./templates/photo-item.hogan');
var loadingTemplate = require('./templates/loading.hogan');
var token;
var username;
var assets;
// Functions

function waitingActivate(id) {
    var el = document.getElementById('waiting-' + id);

    if (el) {
        el.classList.add('active');
    }
}

function waitingDeactivate(id) {
    var el = document.getElementById('waiting-' + id);

    if (el) {
        el.classList.remove('active');
    }
}
function downloadFile(downloadUrl, callback) {
    var xhr = new XMLHttpRequest();
    xhr.open('GET', downloadUrl);
    if ( xhr.overrideMimeType ) xhr.overrideMimeType("text/plain; charset=x-user-defined");
    xhr.onload = function() {
        callback(xhr.responseText);
    };
    xhr.onerror = function() {
        callback(null);
    };
    xhr.send();
}

function ajaxJSONGet(url, callback){
    var http_request = new XMLHttpRequest();
    http_request.open("GET", url, true);
    http_request.onreadystatechange = function () {
        var done = 4;
        var ok = 200;
        if (http_request.readyState === done && http_request.status === ok){
            callback(JSON.parse(http_request.responseText));
        }
    };
    http_request.send();
}

function waitFor(handler, next, timeout) {
    timeout = timeout || 100;
    var result = handler ? handler(): true;

    if (! result) {
        setTimeout(function() {
            waitFor(handler, next, timeout);
        }, timeout);
    } else {
        next && next();
    }
}

function base64Encode(str) {
    var CHARS = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
    var out = "", i = 0, len = str.length, c1, c2, c3;
    while (i < len) {
        c1 = str.charCodeAt(i++) & 0xff;
        if (i == len) {
            out += CHARS.charAt(c1 >> 2);
            out += CHARS.charAt((c1 & 0x3) << 4);
            out += "==";
            break;
        }
        c2 = str.charCodeAt(i++);
        if (i == len) {
            out += CHARS.charAt(c1 >> 2);
            out += CHARS.charAt(((c1 & 0x3)<< 4) | ((c2 & 0xF0) >> 4));
            out += CHARS.charAt((c2 & 0xF) << 2);
            out += "=";
            break;
        }
        c3 = str.charCodeAt(i++);
        out += CHARS.charAt(c1 >> 2);
        out += CHARS.charAt(((c1 & 0x3) << 4) | ((c2 & 0xF0) >> 4));
        out += CHARS.charAt(((c2 & 0xF) << 2) | ((c3 & 0xC0) >> 6));
        out += CHARS.charAt(c3 & 0x3F);
    }
    return out;
}

function openPickDialog() {
    getAlbums();
}

function getAlbums(){
    console.log("get pizap borders albums");
    initDialog();

    var listEl = document.getElementById('photopicker-albums-list');
    while (listEl.firstChild) {
        listEl.removeChild(listEl.firstChild);
    }
    var listEls = document.getElementById('scroll-content');
    while (listEls.firstChild) {
        listEls.removeChild(listEls.firstChild);
    }
    waitingActivate('dialog');

    downloadFile("/assets.json", function(content) {
        assets = JSON.parse(content);
        var listEl = document.getElementById('photopicker-albums-list');
        while (listEl.firstChild) {
            listEl.removeChild(listEl.firstChild);
        }
        var basePath = assets.CDNPath + assets.Backgrounds.Path;
        if('https:' == document.location.protocol){
            basePath = assets.CDNSSLPath + assets.Backgrounds.Path;
        }
        var last = "";

        for(var i = 0; i < assets.Backgrounds.items.length;i++) {
            if(last != assets.Backgrounds.items[i].Name) {
                if(i == 0){
                    getBackgroundPhotos(assets.Backgrounds.items[i].Name, function() {
                        deactivate('dialog-loading');
                    });

                }
                listEl.insertAdjacentHTML('beforeend', sourceItemTemplate.render({
                    id: assets.Backgrounds.items[i].Name,
                    name: assets.Backgrounds.items[i].Name,
                    icon: basePath + assets.Backgrounds.items[i].Thumb
                }));
            }
            last = assets.Backgrounds.items[i].Name;
        }
        waitingDeactivate('dialog');

    });

    activate('dialog');
}
function findParentNode(node, attrName) {
    if (! node) {
        return null;
    }

    while (node) {
        if (node.getAttribute && node.getAttribute(attrName) !== null) {
            break;
        }
        node = node.parentNode;
    }

    return node;
}
function initDialog() {
    var html = dialogTemplate.render();

    var bodyEl = document.getElementsByTagName('BODY')[0];
    var d = document.getElementById("photopicker");
    if(d){
        bodyEl.removeChild(d);
    }
    bodyEl.insertAdjacentHTML('beforeend', html);

    var waitingHtml = loadingTemplate.render();
    var w = document.getElementById("waiting");
    if(w){
        bodyEl.removeChild(w);
    }
    bodyEl.insertAdjacentHTML('beforeend', waitingHtml);

    var cancelBtn = document.getElementById('photopicker-cancel-btn');
    cancelBtn.addEventListener('click', function(event) {
        event.preventDefault();
        event.stopPropagation();

        var listEl = document.getElementById('photopicker-albums-list');
        listEl.removeEventListener('click', albumOnSelected);

        var listEl = document.getElementById('photopicker-album-files');
        listEl.removeEventListener('click', photoOnSelected);

        deactivate('dialog');

        //callback && callback(null);
    }, false);

    var listEl = document.getElementById('photopicker-albums-list');
    listEl.removeEventListener('click', albumOnSelected);
    listEl.addEventListener('click', albumOnSelected);

    var listEl = document.getElementById('photopicker-album-files');
    listEl.removeEventListener('click', photoOnSelected);
    listEl.addEventListener('click', photoOnSelected);
}
function albumOnSelected(event) {
    var target = findParentNode(event.target, 'data-id');

    if (target !== null) {
        event.preventDefault();
        event.stopPropagation();

        var sourceId = target.getAttribute('data-id');
        activate('dialog-loading');
        getBackgroundPhotos(sourceId, function() {
            deactivate('dialog-loading');
        });
    }
}

function photoOnSelected(event) {
    var target = event.target;
    var result = [];

    waitingActivate('dialog');
    if (target !== null && target.getAttribute('data-src')) {
        event.preventDefault();
        event.stopPropagation();
        var listEl = document.getElementById('photopicker-albums-list');
        listEl.removeEventListener('click', albumOnSelected);

        var listEl = document.getElementById('photopicker-album-files');
        listEl.removeEventListener('click', photoOnSelected);

        var source = target.getAttribute('data-src');
        console.log("photo selected");
        console.log(source);
        deactivate('dialog');

        downloadFile(source, (function(file) {
            var ext = file.toLowerCase().match(/\.(jpg|jpeg|png|gif)/);
            ext = typeof ext[1] !== 'undefined' ? ext[1]: null;
            var mimeType = null;

            switch (ext) {
                case 'png': mimeType = 'image/png'; break;
                case 'gif': mimeType = 'image/gif'; break;
                case 'jpg':
                case 'jpeg':
                default: mimeType = 'image/jpeg'; break;
            }

            return function(content) {
                setTimeout(function() {
                    result.push('data:' + mimeType + ';base64,' + base64Encode(content));
                }, 1);
            };
        })(source));

        waitFor(function() {
            return 1 === result.length;
        }, function() {
            waitingDeactivate('dialog');
            parentCallback && parentCallback(result);
        });
    } else {
        waitingDeactivate('dialog');
        parentCallback && parentCallback(null);
    }
}
function zeroPad(num, places) {
    var zero = places - num.toString().length + 1;
    return Array(+(zero > 0 && zero)).join("0") + num;
}
function getBackgroundPhotos(id, cb, removeOld) {
    activate('dialog-loading');
    var listEl = document.getElementById('scroll-content');
    if(removeOld != false) {
        while (listEl.firstChild) {
            listEl.removeChild(listEl.firstChild);
        }
    }

    if(assets && assets.Borders && assets.Borders.items){
        for(var i = 0; i < assets.Borders.items.length;i++) {
            if(assets.Borders.items[i].Name === id){
                var item = assets.Borders.items[i];
                var basePath = assets.CDNPath + assets.Borders.Path + item.Path;
                if('https:' == document.location.protocol){
                    basePath = assets.CDNSSLPath + assets.Borders.Path + item.Path;
                }
                var thumbnailExt = "jpg";
                var thumbnailPrefix = "thumb";
                var thumbnailDigits = 1;
                var pathPrefix = "img";
                var pathExt = "jpg";
                var pathDigits = 1;

                if(item.ThumbnailExt) thumbnailExt = item.ThumbnailExt;
                if(item.ThumbnailPrefix) thumbnailPrefix = item.ThumbnailPrefix;
                if(item.ThumbnailDigits) thumbnailDigits = item.ThumbnailDigits;

                if(item.PathExt) pathExt = item.PathExt;
                if(item.PathPrefix) pathPrefix = item.PathPrefix;
                if(item.PathDigits) pathDigits = item.PathDigits;

                for(var j = 1; j <= item.ItemCount;j++){
                    var thumb = basePath + thumbnailPrefix + zeroPad(j, thumbnailDigits) + "." + thumbnailExt;
                    var source = basePath + pathPrefix + zeroPad(j, pathDigits) + "." + pathExt;

                    var p = {id: id + j, source: source, picture: thumb};
                    listEl.insertAdjacentHTML('beforeend', photoItemTemplate.render(p));
                }
            }
        }

    }
    listEl.style.webkitTransform = 'scale(1)';
    cb && cb();
}

function activate(id) {
    var el = document.getElementById('photopicker-' + id);

    if (el) {
        el.classList.add('active');
    }
}

function deactivate(id) {
    var el = document.getElementById('photopicker-' + id);

    if (el) {
        el.classList.remove('active');
    }
}

// Exports

module.exports = function(options, callback) {

    if (typeof callback !== undefined) {
        parentCallback = callback;
    }

    if (typeof options.multiselect !== 'undefined') {
        multiselect = !! options.multiselect;
    }

    if (typeof options.clientId !== 'undefined') {
        clientId = options.clientId;
    }

    if(typeof options.username !== 'undefined'){
        username = options.username;
    }

    if(typeof options.token !== 'undefined'){
        token = options.token;
    }


    if (! inited) {
        inited = true;
    }

    openPickDialog();
};
}, {"./templates/file-picker.hogan":27,"./templates/asset-folder.hogan":8,"./templates/photo-item.hogan":9,"./templates/loading.hogan":26}]}, {}, {"1":""})